#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networkfirewall_rule_group
short_description: create, delete and modify AWS Network Firewall rule groups
version_added: 4.0.0
description:
  - A module for managing AWS Network Firewall rule groups.
  - U(https://docs.aws.amazon.com/network-firewall/latest/developerguide/index.html)
  - Currently only supports C(stateful) firewall groups.
options:
  arn:
    description:
      - The ARN of the Network Firewall rule group.
      - Exactly one of I(arn) and I(name) must be provided.
    required: false
    type: str
  name:
    description:
      - The name of the Network Firewall rule group.
      - When I(name) is set, I(rule_type) must also be set.
    required: false
    type: str
  rule_type:
    description:
      - Indicates whether the rule group is stateless or stateful.
      - Stateless rulesets are currently not supported.
      - Required if I(name) is set.
    required: false
    aliases: ['type' ]
    choices: ['stateful']
#    choices: ['stateful', 'stateless']
    type: str
  state:
    description:
      - Create or remove the Network Firewall rule group.
    required: false
    choices: ['present', 'absent']
    default: 'present'
    type: str
  capacity:
    description:
      - The maximum operating resources that this rule group can use.
      - Once a rule group is created this parameter is immutable.
      - See also the AWS documentation about how capacityis calculated
        U(https://docs.aws.amazon.com/network-firewall/latest/developerguide/nwfw-rule-group-capacity.html)
      - This option is mandatory when creating a new rule group.
    type: int
    required: false
  rule_order:
    description:
      - Indicates how to manage the order of the rule evaluation for the rule group.
      - Once a rule group is created this parameter is immutable.
      - Mutually exclusive with I(rule_type=stateless).
      - For more information on how rules are evaluated read the AWS documentation
        U(https://docs.aws.amazon.com/network-firewall/latest/developerguide/suricata-rule-evaluation-order.html).
    type: str
    required: false
    choices: ['default', 'strict']
    aliases: ['stateful_rule_order']
  description:
    description:
      - A description of the AWS Network Firewall rule group.
    type: str
  ip_variables:
    description:
      - A dictionary mapping variable names to a list of IP addresses and address ranges, in CIDR notation.
      - For example C({EXAMPLE_HOSTS:["192.0.2.0/24", "203.0.113.42"]}).
      - Mutually exclusive with I(domain_list).
    type: dict
    required: false
    aliases: ['ip_set_variables']
  purge_ip_variables:
    description:
      - Whether to purge variable names not mentioned in the I(ip_variables)
        dictionary.
      - To remove all IP Set Variables it is necessary to explicitly set I(ip_variables={})
        and I(purge_port_variables=true).
    type: bool
    default: true
    required: false
    aliases: ['purge_ip_set_variables']
  port_variables:
    description:
      - A dictionary mapping variable names to a list of ports.
      - For example C({SECURE_PORTS:["22", "443"]}).
    type: dict
    required: false
    aliases: ['port_set_variables']
  purge_port_variables:
    description:
      - Whether to purge variable names not mentioned in the I(port_variables)
        dictionary.
      - To remove all Port Set Variables it is necessary to explicitly set I(port_variables={})
        and I(purge_port_variables=true).
    type: bool
    required: false
    default: true
    aliases: ['purge_port_set_variables']
  rule_strings:
    description:
      - Rules in Suricata format.
      - If I(rule_strings) is specified, it must include at least one entry.
      - For more information read the AWS documentation
        U(https://docs.aws.amazon.com/network-firewall/latest/developerguide/suricata-limitations-caveats.html)
        and the Suricata documentation
        U(https://suricata.readthedocs.io/en/suricata-6.0.0/rules/intro.html).
      - Mutually exclusive with I(rule_type=stateless).
      - Mutually exclusive with I(domain_list) and I(rule_list).
      - Exactly one of I(rule_strings), I(domain_list) or I(rule_list) must be
        specified at creation time.
    type: list
    elements: str
    required: false
  domain_list:
    description:
      - Inspection criteria for a domain list rule group.
      - When set overwrites all Domain List settings with the new configuration.
      - For more information about domain name based filtering
        read the AWS documentation
        U(https://docs.aws.amazon.com/network-firewall/latest/developerguide/stateful-rule-groups-domain-names.html).
      - Mutually exclusive with I(rule_type=stateless).
      - Mutually exclusive with I(ip_variables), I(rule_list) and I(rule_strings).
      - Exactly one of I(rule_strings), I(domain_list) or I(rule_list) must be
        specified at creation time.
    type: dict
    required: false
    suboptions:
      domain_names:
        description:
          - A list of domain names to look for in the traffic flow.
        type: list
        elements: str
        required: true
      filter_http:
        description:
          - Whether HTTP traffic should be inspected (uses the host header).
        type: bool
        required: false
        default: false
      filter_https:
        description:
          - Whether HTTPS traffic should be inspected (uses the SNI).
        type: bool
        required: false
        default: false
      action:
        description:
          - Action to perform on traffic that matches the rule match settings.
        type: str
        required: true
        choices: ['allow', 'deny']
      source_ips:
        description:
          - Used to expand the local network definition beyond the CIDR range
            of the VPC where you deploy Network Firewall.
        type: list
        elements: str
        required: false
  rule_list:
    description:
      - Inspection criteria to be used for a 5-tuple based rule group.
      - When set overwrites all existing 5-tuple rules with the new configuration.
      - Mutually exclusive with I(domain_list) and I(rule_strings).
      - Mutually exclusive with I(rule_type=stateless).
      - Exactly one of I(rule_strings), I(domain_list) or I(rule_list) must be
        specified at creation time.
      - For more information about valid values see the AWS documentation
        U(https://docs.aws.amazon.com/network-firewall/latest/APIReference/API_StatefulRule.html)
        and
        U(https://docs.aws.amazon.com/network-firewall/latest/APIReference/API_Header.html).
      - 'Note: Idempotency when comparing AWS Web UI and Ansiible managed rules can not be guaranteed'
    type: list
    elements: dict
    required: false
    aliases: ['stateful_rule_list']
    suboptions:
      action:
        description:
          - What Network Firewall should do with the packets in a traffic flow when the flow matches.
        type: str
        required: true
        choices: ['pass', 'drop', 'alert']
      protocol:
        description:
          - The protocol to inspect for. To specify all, you can use C(IP), because all traffic on AWS is C(IP).
        type: str
        required: true
      source:
        description:
          - The source IP address or address range to inspect for, in CIDR notation.
          - To match with any address, specify C(ANY).
        type: str
        required: true
      source_port:
        description:
          - The source port to inspect for.
          - To match with any port, specify C(ANY).
        type: str
        required: true
      direction:
        description:
          - The direction of traffic flow to inspect.
          - If set to C(any), the inspection matches both traffic going from the
            I(source) to the I(destination) and from the I(destination) to the
            I(source).
          - If set to C(forward), the inspection only matches traffic going from the
            I(source) to the I(destination).
        type: str
        required: false
        default: 'forward'
        choices: ['forward', 'any']
      destination:
        description:
          - The destination IP address or address range to inspect for, in CIDR notation.
          - To match with any address, specify C(ANY).
        type: str
        required: true
      destination_port:
        description:
          - The source port to inspect for.
          - To match with any port, specify C(ANY).
        type: str
        required: true
      sid:
        description:
          - The signature ID of the rule.
          - A unique I(sid) must be passed for all rules.
        type: int
        required: true
      rule_options:
        description:
          - Additional options for the rule.
          - 5-tuple based rules are converted by AWS into Suricata rules, for more
            complex options requirements where order matters consider using I(rule_strings).
          - A dictionary mapping Suricata RuleOptions names to a list of values.
          - The examples section contains some examples of using rule_options.
          - For more information read the AWS documentation
            U(https://docs.aws.amazon.com/network-firewall/latest/developerguide/suricata-limitations-caveats.html)
            and the Suricata documentation
            U(https://suricata.readthedocs.io/en/suricata-6.0.0/rules/intro.html).
        type: dict
        required: false
  wait:
    description:
      - Whether to wait for the firewall rule group to reach the
        C(ACTIVE) or C(DELETED) state before the module returns.
    type: bool
    required: false
    default: true
  wait_timeout:
    description:
      - Maximum time, in seconds, to wait for the firewall rule group
        to reach the expected state.
      - Defaults to 600 seconds.
    type: int
    required: false

author:
  - Mark Chappell (@tremble)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Create a rule group
- name: Create a minimal AWS Network Firewall Rule Group
  community.aws.networkfirewall_rule_group:
    name: 'MinimalGroup'
    type: 'stateful'
    capacity: 200
    rule_strings:
      - 'pass tcp any any -> any any (sid:1000001;)'

# Create an example rule group using rule_list
- name: Create 5-tuple Rule List based rule group
  community.aws.networkfirewall_rule_group:
    name: 'ExampleGroup'
    type: 'stateful'
    description: 'My description'
    rule_order: default
    capacity: 100
    rule_list:
      - sid: 1
        direction: forward
        action: pass
        protocol: IP
        source: any
        source_port: any
        destination: any
        destination_port: any

# Create an example rule group using rule_list
- name: Create 5-tuple Rule List based rule group
  community.aws.networkfirewall_rule_group:
    name: 'ExampleGroup'
    type: 'stateful'
    description: 'My description'
    ip_variables:
      SOURCE_IPS: ['203.0.113.0/24', '198.51.100.42']
      DESTINATION_IPS: ['192.0.2.0/24', '198.51.100.48']
    port_variables:
      HTTP_PORTS: [80, 8080]
    rule_order: default
    capacity: 100
    rule_list:
      # Allow 'Destination Unreachable' traffic
      - sid: 1
        action: pass
        protocol: icmp
        source: any
        source_port: any
        destination: any
        destination_port: any
        rule_options:
          itype: 3
      - sid: 2
        action: drop
        protocol: tcp
        source: "$SOURCE_IPS"
        source_port: any
        destination: "$DESTINATION_IPS"
        destination_port: "$HTTP_PORTS"
        rule_options:
          urilen: ["20<>40"]
          # Where only a keyword is needed, add the keword, but no value
          http_uri:
          # Settings where Suricata expects raw strings (like the content
          # keyword) will need to have the double-quotes explicitly escaped and
          # passed because there's no practical way to distinguish between them
          # and flags.
          content: '"index.php"'

# Create an example rule group using Suricata rule strings
- name: Create Suricata rule string based rule group
  community.aws.networkfirewall_rule_group:
    name: 'ExampleSuricata'
    type: 'stateful'
    description: 'My description'
    capacity: 200
    ip_variables:
      EXAMPLE_IP: ['203.0.113.0/24', '198.51.100.42']
      ANOTHER_EXAMPLE: ['192.0.2.0/24', '198.51.100.48']
    port_variables:
      EXAMPLE_PORT: [443, 22]
    rule_strings:
      - 'pass tcp any any -> $EXAMPLE_IP $EXAMPLE_PORT (sid:1000001;)'
      - 'pass udp any any -> $ANOTHER_EXAMPLE any (sid:1000002;)'

# Create an example Domain List based rule group
- name: Create Domain List based rule group
  community.aws.networkfirewall_rule_group:
    name: 'ExampleDomainList'
    type: 'stateful'
    description: 'My description'
    capacity: 100
    domain_list:
      domain_names:
        - 'example.com'
        - '.example.net'
      filter_https: true
      filter_http: true
      action: allow
      source_ips: '192.0.2.0/24'

# Update the description of a rule group
- name: Update the description of a rule group
  community.aws.networkfirewall_rule_group:
    name: 'MinimalGroup'
    type: 'stateful'
    description: 'Another description'

# Update IP Variables for a rule group
- name: Update IP Variables
  community.aws.networkfirewall_rule_group:
    name: 'ExampleGroup'
    type: 'stateful'
    ip_variables:
      EXAMPLE_IP: ['192.0.2.0/24', '203.0.113.0/24', '198.51.100.42']
    purge_ip_variables: false

# Delete a rule group
- name: Delete a rule group
  community.aws.networkfirewall_rule_group:
    name: 'MinimalGroup'
    type: 'stateful'
    state: absent
"""

RETURN = r"""
rule_group:
  description: Details of the rules in the rule group
  type: dict
  returned: success
  contains:
    rule_group:
      description: Details of the rules in the rule group
      type: dict
      returned: success
      contains:
        rule_variables:
          description: Settings that are available for use in the rules in the rule group.
          returned: When rule variables are attached to the rule group.
          type: complex
          contains:
            ip_sets:
              description: A dictionary mapping variable names to IP addresses in CIDR format.
              returned: success
              type: dict
              example: ['192.0.2.0/24']
            port_sets:
              description: A dictionary mapping variable names to ports
              returned: success
              type: dict
              example: ['42']
        stateful_rule_options:
          description: Additional options governing how Network Firewall handles stateful rules.
          returned: When the rule group is either "rules string" or "rules list" based.
          type: dict
          contains:
            rule_order:
              description: The order in which rules will be evaluated.
              returned: success
              type: str
              example: 'DEFAULT_ACTION_ORDER'
        rules_source:
          description: Inspection criteria used for a 5-tuple based rule group.
          returned: success
          type: dict
          contains:
            stateful_rules:
              description: A list of dictionaries describing the rules that the rule group is comprised of.
              returned: When the rule group is "rules list" based.
              type: list
              elements: dict
              contains:
                action:
                  description: What action to perform when a flow matches the rule criteria.
                  returned: success
                  type: str
                  example: 'PASS'
                header:
                  description: A description of the criteria used for the rule.
                  returned: success
                  type: dict
                  contains:
                    protocol:
                      description: The protocol to inspect for.
                      returned: success
                      type: str
                      example: 'IP'
                    source:
                      description: The source address or range of addresses to inspect for.
                      returned: success
                      type: str
                      example: '203.0.113.98'
                    source_port:
                      description: The source port to inspect for.
                      returned: success
                      type: str
                      example: '42'
                    destination:
                      description: The destination address or range of addresses to inspect for.
                      returned: success
                      type: str
                      example: '198.51.100.0/24'
                    destination_port:
                      description: The destination port to inspect for.
                      returned: success
                      type: str
                      example: '6666:6667'
                    direction:
                      description: The direction of traffic flow to inspect.
                      returned: success
                      type: str
                      example: 'FORWARD'
                rule_options:
                  description: Additional Suricata RuleOptions settings for the rule.
                  returned: success
                  type: list
                  elements: dict
                  contains:
                    keyword:
                      description: The keyword for the setting.
                      returned: success
                      type: str
                      example: 'sid:1'
                    settings:
                      description: A list of values passed to the setting.
                      returned: When values are available
                      type: list
                      elements: str
            rules_string:
              description: A string describing the rules that the rule group is comprised of.
              returned: When the rule group is "rules string" based.
              type: str
            rules_source_list:
              description: A description of the criteria for a domain list rule group.
              returned: When the rule group is "domain list" based.
              type: dict
              contains:
                targets:
                  description: A list of domain names to be inspected for.
                  returned: success
                  type: list
                  elements: str
                  example: ['abc.example.com', '.example.net']
                target_types:
                  description: The protocols to be inspected by the rule group.
                  returned: success
                  type: list
                  elements: str
                  example: ['TLS_SNI', 'HTTP_HOST']
                generated_rules_type:
                  description: Whether the rule group allows or denies access to the domains in the list.
                  returned: success
                  type: str
                  example: 'ALLOWLIST'
            stateless_rules_and_custom_actions:
              description: A description of the criteria for a stateless rule group.
              returned: When the rule group is a stateless rule group.
              type: dict
              contains:
                stateless_rules:
                  description: A list of stateless rules for use in a stateless rule group.
                  type: list
                  elements: dict
                  contains:
                    rule_definition:
                      description: Describes the stateless 5-tuple inspection criteria and actions for the rule.
                      returned: success
                      type: dict
                      contains:
                        match_attributes:
                          description: Describes the stateless 5-tuple inspection criteria for the rule.
                          returned: success
                          type: dict
                          contains:
                            sources:
                              description: The source IP addresses and address ranges to inspect for.
                              returned: success
                              type: list
                              elements: dict
                              contains:
                                address_definition:
                                  description: An IP address or a block of IP addresses in CIDR notation.
                                  returned: success
                                  type: str
                                  example: '192.0.2.3'
                            destinations:
                              description: The destination IP addresses and address ranges to inspect for.
                              returned: success
                              type: list
                              elements: dict
                              contains:
                                address_definition:
                                  description: An IP address or a block of IP addresses in CIDR notation.
                                  returned: success
                                  type: str
                                  example: '192.0.2.3'
                            source_ports:
                              description: The source port ranges to inspect for.
                              returned: success
                              type: list
                              elements: dict
                              contains:
                                from_port:
                                  description: The lower limit of the port range.
                                  returned: success
                                  type: int
                                to_port:
                                  description: The upper limit of the port range.
                                  returned: success
                                  type: int
                            destination_ports:
                              description: The destination port ranges to inspect for.
                              returned: success
                              type: list
                              elements: dict
                              contains:
                                from_port:
                                  description: The lower limit of the port range.
                                  returned: success
                                  type: int
                                to_port:
                                  description: The upper limit of the port range.
                                  returned: success
                                  type: int
                            protocols:
                              description: The IANA protocol numbers of the protocols to inspect for.
                              returned: success
                              type: list
                              elements: int
                              example: [6]
                            tcp_flags:
                              description: The TCP flags and masks to inspect for.
                              returned: success
                              type: list
                              elements: dict
                              contains:
                                flags:
                                  description: Used with masks to define the TCP flags that flows are inspected for.
                                  returned: success
                                  type: list
                                  elements: str
                                masks:
                                  description: The set of flags considered during inspection.
                                  returned: success
                                  type: list
                                  elements: str
                        actions:
                          description: The actions to take when a flow matches the rule.
                          returned: success
                          type: list
                          elements: str
                          example: ['aws:pass', 'CustomActionName']
                    priority:
                      description: Indicates the order in which to run this rule relative to all of the rules that are defined for a stateless rule group.
                      returned: success
                      type: int
                custom_actions:
                  description: A list of individual custom action definitions that are available for use in stateless rules.
                  type: list
                  elements: dict
                  contains:
                    action_name:
                      description: The name for the custom action.
                      returned: success
                      type: str
                    action_definition:
                      description: The custom action associated with the action name.
                      returned: success
                      type: dict
                      contains:
                        publish_metric_action:
                          description: The description of an action which publishes to CloudWatch.
                          returned: When the action publishes to CloudWatch.
                          type: dict
                          contains:
                            dimensions:
                              description: The value to use in an Amazon CloudWatch custom metric dimension.
                              returned: success
                              type: list
                              elements: dict
                              contains:
                                value:
                                  description: The value to use in the custom metric dimension.
                                  returned: success
                                  type: str
    rule_group_metadata:
      description: Details of the rules in the rule group
      type: dict
      returned: success
      contains:
        capacity:
          description: The maximum operating resources that this rule group can use.
          type: int
          returned: success
        consumed_capacity:
          description: The number of capacity units currently consumed by the rule group rules.
          type: int
          returned: success
        description:
          description: A description of the rule group.
          type: str
          returned: success
        number_of_associations:
          description: The number of firewall policies that use this rule group.
          type: int
          returned: success
        rule_group_arn:
          description: The ARN for the rule group
          type: int
          returned: success
          example: 'arn:aws:network-firewall:us-east-1:123456789012:stateful-rulegroup/ExampleGroup'
        rule_group_id:
          description: A unique identifier for the rule group.
          type: int
          returned: success
          example: '12345678-abcd-1234-abcd-123456789abc'
        rule_group_name:
          description: The name of the rule group.
          type: str
          returned: success
        rule_group_status:
          description: The current status of a rule group.
          type: str
          returned: success
          example: 'DELETING'
        tags:
          description: A dictionary representing the tags associated with the rule group.
          type: dict
          returned: success
        type:
          description: Whether the rule group is stateless or stateful.
          type: str
          returned: success
          example: 'STATEFUL'
"""

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule
from ansible_collections.community.aws.plugins.module_utils.networkfirewall import NetworkFirewallRuleManager


def main():
    domain_list_spec = dict(
        domain_names=dict(type="list", elements="str", required=True),
        filter_http=dict(type="bool", required=False, default=False),
        filter_https=dict(type="bool", required=False, default=False),
        action=dict(type="str", required=True, choices=["allow", "deny"]),
        source_ips=dict(type="list", elements="str", required=False),
    )

    rule_list_spec = dict(
        action=dict(type="str", required=True, choices=["pass", "drop", "alert"]),
        protocol=dict(type="str", required=True),
        source=dict(type="str", required=True),
        source_port=dict(type="str", required=True),
        direction=dict(type="str", required=False, default="forward", choices=["forward", "any"]),
        destination=dict(type="str", required=True),
        destination_port=dict(type="str", required=True),
        sid=dict(type="int", required=True),
        rule_options=dict(type="dict", required=False),
    )

    argument_spec = dict(
        arn=dict(type="str", required=False),
        name=dict(type="str", required=False),
        rule_type=dict(type="str", required=False, aliases=["type"], choices=["stateful"]),
        # rule_type=dict(type='str', required=True, aliases=['type'], choices=['stateless', 'stateful']),
        state=dict(type="str", required=False, choices=["present", "absent"], default="present"),
        capacity=dict(type="int", required=False),
        rule_order=dict(type="str", required=False, aliases=["stateful_rule_order"], choices=["default", "strict"]),
        description=dict(type="str", required=False),
        ip_variables=dict(type="dict", required=False, aliases=["ip_set_variables"]),
        purge_ip_variables=dict(type="bool", required=False, aliases=["purge_ip_set_variables"], default=True),
        port_variables=dict(type="dict", required=False, aliases=["port_set_variables"]),
        purge_port_variables=dict(type="bool", required=False, aliases=["purge_port_set_variables"], default=True),
        rule_strings=dict(type="list", elements="str", required=False),
        domain_list=dict(type="dict", options=domain_list_spec, required=False),
        rule_list=dict(
            type="list", elements="dict", aliases=["stateful_rule_list"], options=rule_list_spec, required=False
        ),
        tags=dict(type="dict", required=False, aliases=["resource_tags"]),
        purge_tags=dict(type="bool", required=False, default=True),
        wait=dict(type="bool", required=False, default=True),
        wait_timeout=dict(type="int", required=False),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        mutually_exclusive=[
            ["name", "arn"],
            ["rule_strings", "domain_list", "rule_list"],
            ["domain_list", "ip_variables"],
        ],
        required_together=[
            ["name", "rule_type"],
        ],
        required_one_of=[
            ["name", "arn"],
        ],
    )

    state = module.params.get("state")
    name = module.params.get("name")
    arn = module.params.get("arn")
    rule_type = module.params.get("rule_type")

    if rule_type == "stateless":
        if module.params.get("rule_order"):
            module.fail_json("rule_order can not be set for stateless rule groups")
        if module.params.get("rule_strings"):
            module.fail_json("rule_strings can only be used for stateful rule groups")
        if module.params.get("rule_list"):
            module.fail_json("rule_list can only be used for stateful rule groups")
        if module.params.get("domain_list"):
            module.fail_json("domain_list can only be used for stateful rule groups")

    manager = NetworkFirewallRuleManager(module, arn=arn, name=name, rule_type=rule_type)
    manager.set_wait(module.params.get("wait", None))
    manager.set_wait_timeout(module.params.get("wait_timeout", None))

    if state == "absent":
        manager.delete()
    else:
        manager.set_description(module.params.get("description"))
        manager.set_capacity(module.params.get("capacity"))
        manager.set_rule_order(module.params.get("rule_order"))
        manager.set_ip_variables(module.params.get("ip_variables"), module.params.get("purge_ip_variables"))
        manager.set_port_variables(module.params.get("port_variables"), module.params.get("purge_port_variables"))
        manager.set_rule_string(module.params.get("rule_strings"))
        manager.set_domain_list(module.params.get("domain_list"))
        manager.set_rule_list(module.params.get("rule_list"))
        manager.set_tags(module.params.get("tags"), module.params.get("purge_tags"))

        manager.flush_changes()

    results = dict(
        changed=manager.changed,
        rule_group=manager.updated_resource,
    )
    if manager.changed:
        diff = dict(
            before=manager.original_resource,
            after=manager.updated_resource,
        )
        results["diff"] = diff
    module.exit_json(**results)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function


__metaclass__ = type

# Copyright: (c) 2015, Peter Sprygada <psprygada@ansible.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


class ModuleDocFragment(object):
    # Standard files documentation fragment
    DOCUMENTATION = r"""options: {}
notes:
  - For information on using CLI and NX-API see the :ref:`NXOS Platform Options guide
    <nxos_platform_options>`
  - For more information on using Ansible to manage network devices see the :ref:`Ansible
    Network Guide <network_guide>`
  - For more information on using Ansible to manage Cisco devices see the `Cisco integration
    page <https://www.ansible.com/integrations/networks/cisco>`_.
"""



#!/usr/bin/python
#
# Copyright (c) 2019 Yunge Zhu, (@yungezz)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_roledefinition_info
version_added: "0.1.2"
short_description: Get Azure Role Definition facts
description:
    - Get facts of Azure Role Definition.

options:
    scope:
        description:
            - The scope of role definition.
        required: True
        type: str
    id:
        description:
            - Role definition id.
        type: str
    role_name:
        description:
            - Role name.
        type: str
    type:
        description:
            - Type of role.
        type: str
        choices:
            - system
            - custom

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Yunge Zhu(@yungezz)

'''

EXAMPLES = '''
- name: List Role Definitions in scope
  azure_rm_roledefinition_info:
    scope: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup

- name: Get Role Definition by name
  azure_rm_roledefinition_info:
    scope: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup
    role_name: myRoleDefinition
'''

RETURN = '''
roledefinitions:
    description:
        - A list of Role Definition facts.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Role Definition ID.
            returned: always
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/providers/Microsoft.Authorization/roleDefinitions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
        role_name:
            description:
                - Role name.
            returned: always
            type: str
            sample: myCustomRoleDefinition
        name:
            description:
                - System assigned role name.
            returned: always
            type: str
            sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
        assignable_scopes:
            description:
                - List of assignable scopes of this definition.
            returned: always
            type: list
            sample: [ "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup" ]
        permissions:
            description:
                - List of Role Definition permissions.
            returned: always
            type: complex
            contains:
                actions:
                    description:
                        - List of allowed actions.
                    returned: always
                    type: list
                    sample: [ 'Microsoft.Compute/virtualMachines/read' ]
                not_actions:
                    description:
                        - List of denied actions.
                    returned: always
                    type: list
                    sample: [ 'Microsoft.Compute/virtualMachines/write' ]
                data_actions:
                    description:
                        - List of allowed data actions.
                    returned: always
                    type: list
                    sample: [ 'Microsoft.Storage/storageAccounts/blobServices/containers/blobs/read' ]
                not_data_actions:
                    description:
                        - List of denied data actions.
                    returned: always
                    type: list
                    sample: [ 'Microsoft.Storage/storageAccounts/blobServices/containers/blobs/write' ]
'''

try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
    from azure.mgmt.authorization import AuthorizationManagementClient

except ImportError:
    # This is handled in azure_rm_common
    pass


def roledefinition_to_dict(role):
    result = dict(
        id=role.id,
        name=role.name,
        type=role.role_type,
        assignable_scopes=role.assignable_scopes,
        description=role.description,
        role_name=role.role_name
    )
    if role.permissions:
        result['permissions'] = [dict(
            actions=p.actions,
            not_actions=p.not_actions,
            data_actions=p.data_actions,
            not_data_actions=p.not_data_actions
        ) for p in role.permissions]
    return result


class AzureRMRoleDefinitionInfo(AzureRMModuleBase):
    def __init__(self):
        self.module_arg_spec = dict(
            scope=dict(
                type='str',
                required=True
            ),
            role_name=dict(type='str'),
            id=dict(type='str'),
            type=dict(
                type='str',
                choices=['custom', 'system']
            ),
        )

        self.role_name = None
        self.scope = None
        self.id = None
        self.type = None

        self.results = dict(
            changed=False
        )

        self._client = None

        super(AzureRMRoleDefinitionInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                        supports_check_mode=True,
                                                        supports_tags=False)

    def exec_module(self, **kwargs):
        """Main module execution method"""
        is_old_facts = self.module._name == 'azure_rm_roledefinition_facts'
        if is_old_facts:
            self.module.deprecate("The 'azure_rm_roledefinition_facts' module has been renamed to 'azure_rm_roledefinition_info'", version=(2.9, ))

        for key in list(self.module_arg_spec.keys()):
            if hasattr(self, key):
                setattr(self, key, kwargs[key])

        if self.type:
            self.type = self.get_role_type(self.type)

        # get management client
        self._client = self.get_mgmt_svc_client(AuthorizationManagementClient,
                                                base_url=self._cloud_environment.endpoints.resource_manager,
                                                is_track2=True,
                                                api_version="2018-01-01-preview")

        if self.id:
            self.results['roledefinitions'] = self.get_by_id()
        elif self.role_name:
            self.results['roledefinitions'] = self.get_by_role_name()
        else:
            self.results['roledefinitions'] = self.list()

        return self.results

    def get_role_type(self, role_type):
        if role_type:
            if role_type == 'custom':
                return 'CustomRole'
            else:
                return 'SystemRole'
        return role_type

    def list(self):
        '''
        List Role Definition in scope.

        :return: deserialized Role Definition state dictionary
        '''
        self.log("List Role Definition in scope {0}".format(self.scope))

        response = []

        try:
            response = list(self._client.role_definitions.list(scope=self.scope))

            if len(response) > 0:
                self.log("Response : {0}".format(response))
                roles = []

                if self.type:
                    roles = [r for r in response if r.role_type == self.type]
                else:
                    roles = response

                if len(roles) > 0:
                    return [roledefinition_to_dict(r) for r in roles]

        except Exception as ex:
            self.log("Didn't find role definition in scope {0}".format(self.scope))

        return response

    def get_by_id(self):
        '''
        Get Role Definition in scope by id.

        :return: deserialized Role Definition state dictionary
        '''
        self.log("Get Role Definition by id {0}".format(self.id))

        response = None

        try:
            response = self._client.role_definitions.get(scope=self.scope, role_definition_id=self.id)
            if response:
                response = roledefinition_to_dict(response)
                if self.type:
                    if response.role_type == self.type:
                        return [response]
                else:
                    return [response]

        except Exception as ex:
            self.log("Didn't find role definition by id {0}".format(self.id))

        return []

    def get_by_role_name(self):
        '''
        Get Role Definition in scope by role name.

        :return: deserialized role definition state dictionary
        '''
        self.log("Get Role Definition by name {0}".format(self.role_name))

        response = []

        try:
            response = self.list()

            if len(response) > 0:
                roles = []
                for r in response:
                    if r['role_name'] == self.role_name:
                        roles.append(r)

                if len(roles) == 1:
                    self.log("Role Definition : {0} found".format(self.role_name))
                    return roles
                if len(roles) > 1:
                    self.fail("Found multiple Role Definitions with name: {0}".format(self.role_name))

        except Exception as ex:
            self.log("Didn't find Role Definition by name {0}".format(self.role_name))

        return []


def main():
    """Main execution"""
    AzureRMRoleDefinitionInfo()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Bgp_address_family parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


def _tmplt_aggregate_address(aggaddr):
    cmd = "aggregate-address {value}"

    if aggaddr.get("as_set"):
        cmd += " as-set"
    if aggaddr.get("as_confed_set"):
        cmd += " as-confed-set"
    if aggaddr.get("summary_only"):
        cmd += " summary-only"
    if aggaddr.get("route_policy"):
        cmd += " route-policy {route_policy}"

    return cmd.format(**aggaddr)


def _tmpl_allocate_label(config_data):
    if "allocate_label" in config_data:
        command = "allocate-label"
        if "all" in config_data["allocate_label"]:
            command += " all"

        if "route_policy" in config_data["allocate_label"]:
            command += " route-policy {route_policy}".format(**config_data["route_policy"])

        return command


def _tmpl_bgp_origin_as_validation(config_data):
    origin_as_conf = config_data.get("bgp", {}).get("origin_as", {}).get("validation")
    if origin_as_conf:
        command = []
        if "disable" in origin_as_conf:
            command.append("bgp origin-as validation disable")
        if "ibgp" in origin_as_conf.get("signal", {}):
            command.append("bgp origin-as validation signal ibgp")

        return command


def _tmpl_bgp_dampening(config_data):
    dampening_conf = config_data.get("bgp", {}).get("dampening", {})
    if dampening_conf:
        command = "bgp dampening"
        if "value" in dampening_conf:
            command += " " + str(dampening_conf["value"])
        if "route_policy" in dampening_conf:
            command += " route-policy " + dampening_conf["route_policy"]

        return command


def _tmpl_maximum_paths_ibgp(config_data):
    ibgp_conf = config_data.get("maximum_paths", {}).get("ibgp", {})
    if ibgp_conf:
        command = "maximum-paths ibgp"
        if "max_path_value" in ibgp_conf:
            command += " " + str(ibgp_conf["max_path_value"])
        if "order_igp_metric" in ibgp_conf:
            command += " order igp-metric"
        elif "selective_order_igp_metric" in ibgp_conf:
            command += " selective order igp-metric"
        elif "set" in ibgp_conf.get("unequal_cost", {}):
            command += " unequal-cost"
            if "order_igp_metric" in ibgp_conf.get("unequal_cost", {}):
                command += " order igp-metric"
            elif "selective_order_igp_metric" in ibgp_conf.get(
                "unequal_cost",
                {},
            ):
                command += " selective order igp-metric"
        return command


def _tmpl_maximum_paths_ebgp(config_data):
    ebgp_conf = config_data.get("maximum_paths", {}).get("ebgp", {})
    if ebgp_conf:
        command = "maximum-paths ebgp"
        if "max_path_value" in ebgp_conf:
            command += " " + str(ebgp_conf["max_path_value"])
        if "order_igp_metric" in ebgp_conf:
            command += " order igp-metric"
        elif "selective_order_igp_metric" in ebgp_conf:
            command += " selective order igp-metric"
        return command


def _tmpl_maximum_paths_eibgp(config_data):
    eibgp_conf = config_data.get("maximum_paths", {}).get("eibgp", {})
    if eibgp_conf:
        command = "maximum-paths ebgp"
        if "max_path_value" in eibgp_conf:
            command += " " + str(eibgp_conf["max_path_value"])
        if "order_igp_metric" in eibgp_conf:
            command += " order igp-metric"
        elif "selective_order_igp_metric" in eibgp_conf:
            command += " selective order igp-metric"
        return command


def _tmpl_network(config_data):
    cmd = "network {network}"
    if config_data.get("backdoor_route_policy"):
        cmd += " backdoor-route-policy {backdoor-route-policy}"
    if config_data.get("route_policy"):
        cmd += " route-policy {route_policy}"
    return cmd.format(**config_data)


def _tmpl_nexthop(config_data):
    nexthop_conf = config_data.get("nexthop", {})
    commands = []
    if nexthop_conf:
        if "resolution_prefix_length_minimum" in nexthop_conf:
            command = "nexthop resolution prefix-length minimum " + str(
                nexthop_conf["resolution_prefix_length_minimum"],
            )
            commands.append(command)
        if "trigger_delay_critical" in nexthop_conf:
            command = "nexthop trigger-delay critical " + str(
                nexthop_conf["trigger_delay_non_critical"],
            )
            commands.append(command)
        if "trigger_delay_non_critical" in nexthop_conf:
            command = "nexthop trigger-delay non-critical " + str(
                nexthop_conf["trigger_delay_non_critical"],
            )
            commands.append(command)
        if "route_policy" in nexthop_conf:
            command += " route-policy " + nexthop_conf["route_policy"]

    return commands


def _tmpl_optimal_route(config_data):
    orr_conf = config_data.get("optimal_route_reflection", {})
    if orr_conf:
        command = "optimal-route-reflection"
        if "group_name" in orr_conf:
            command += " " + str(orr_conf["value"])
        if "primary_address" in orr_conf:
            command += " " + orr_conf["primary_address"]
        if "secondary_address" in orr_conf:
            command += " " + orr_conf["secondary_address"]
        return command


def _tmpl_update(config_data):
    update_conf = config_data.get("update", {})
    update_wait = config_data.get("update", {}).get("wait_install")
    update_limit = config_data.get("update", {}).get("limit", {})
    commands = []
    if update_conf:
        if update_wait:
            command = "update wait-install"
            commands.append(command)
        if "address_family" in update_limit:
            command = "update limit address-family " + str(
                update_limit["address_family"],
            )
            commands.append(command)
        if "sub_group" in update_limit:
            if "ibgp" in update_limit["sub_group"]:
                command = "update limit sub-group ibgp " + str(
                    update_limit["sub_group"]["ibgp"],
                )
                commands.append(command)
            if "ebgp" in update_limit["sub_group"]:
                command = "update limit sub-group ebgp " + str(
                    update_limit["sub_group"]["ebgp"],
                )
                commands.append(command)
    return commands


def _tmplt_redistribute(redis):
    command = "redistribute {protocol}".format(**redis)
    if redis.get("id"):
        command += " {id}".format(**redis)
    if redis.get("metric"):
        command += " metric {metric}".format(**redis)
    if redis.get("level"):
        command += " level {level}".format(**redis)
    if redis.get("internal"):
        command += " internal"
    if redis.get("external"):
        command += " external"
    if redis.get("nssa_external"):
        command += " nssa-external"
    if redis.get("external_ospf"):
        command += " external {external_ospf}".format(**redis)
    if redis.get("route_policy"):
        command += " route-policy {route_policy}".format(**redis)
    return command


def _tmpl_vrf_all(config_data):
    conf = config_data.get("vrf_all", {})
    commands = []
    if conf:
        if "source_rt_import_policy" in conf:
            commands.append("vrf all source rt import-policy")
        if "label_mode" in conf:
            command = "vrf all label mode"
            if "per_ce" in conf.get("label_mode"):
                command += " per-ce"
            elif "per_vrf" in conf.get("label_mode"):
                command += " per-vrf"
            elif "route_policy" in conf.get("label_mode"):
                command += " route-policy " + conf["route_policy"]
        if "table_policy" in conf:
            command = "vrf all table-policy " + conf["table_policy"]
            commands.append(command)
        return command


def _tmpl_wt(config_data):
    conf = config_data.get("weight", "")
    if conf:
        command = "weight"
        if "reset_on_import" in conf:
            command += " reset-on-import"
        elif "reset_on_import_disable" in conf:
            command += " reset-on-import disable"
        return command


def _tmpl_label_mode(conf):
    if "label_mode" in conf:
        command = "vrf all label mode"
        if "per_ce" in conf.get("label_mode"):
            command += " per-ce"
        elif "per_vrf" in conf.get("label_mode"):
            command += " per-vrf"
        elif "per_prefix" in conf.get("label_mode"):
            command += " per-prefix"
        elif "route_policy" in conf.get("label_mode"):
            command += " route-policy " + conf["route_policy"]
    return command


class Bgp_address_familyTemplate(NetworkTemplate):
    def __init__(self, lines=None):
        super(Bgp_address_familyTemplate, self).__init__(
            lines=lines,
            tmplt=self,
        )

    # fmt: off
    PARSERS = [
        {
            "name": "router",
            "getval": re.compile(
                r"""
                ^router\s
                bgp
                \s(?P<as_num>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "router bgp {{ as_number }}",
            "result": {"as_number": "{{ as_num }}"},
            "shared": True,
        },
        {
            "name": "vrf",
            "getval": re.compile(
                r"""
                \s+vrf
                \s(?P<vrf>\b(?!all\b)\S+)$""",
                re.VERBOSE,
            ),
            "setval": "vrf {{ vrf }}",
            "result": {
            },
            "shared": True,
        },
        {
            "name": "address_family",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\b(?!all\b)\S+))?
                (?P<address_family>\s+address-family\s(?P<afi>\S+)\s(?P<safi>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "address-family {{ afi}} {{safi}}",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "afi": "{{ afi}}",
                        "safi": "{{safi}}",
                        "vrf": "{{ vrf }}",
                    },
                },
            },
            "shared": True,
        },
        {
            "name": "advertise_best_external",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+advertise\s(?P<abe>best-external)
                $""", re.VERBOSE,
            ),
            "setval": "advertise best-external",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "advertise_best_external": "{{True if abe is defined}}",
                    },
                },
            },
        },
        {
            "name": "allocate_label",
            "getval": re.compile(
                r"""
                \s+allocate-label\s(?P<all>all)
                 (\sroute-policy\s(?P<route_policy>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_allocate_label,
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "allocate_label": {
                            "all": "{{True if all is defined}}",
                            "route_policy": "{{route_policy}}",
                        },
                    },
                },
            },
        },

        {
            "name": "aggregate_address",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+aggregate-address\s(?P<value>\S+)
                (\sas-set(?P<as_set>))?
                (\sas-confed-set(?P<as_confed_set>))?
                (\ssummary-only(?P<summery_only>))?
                (\sroute-policy\s(?P<route_policy>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": _tmplt_aggregate_address,
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "aggregate_address": [
                            {
                                "as_set": "{{True if as_set is defined}}",
                                "as_confed_set": "{{True if as_confed_set is defined}}",
                                "summary_only": "{{True if summery_only is defined}}",
                                "value": "{{value}}",
                                "route_policy": "{{route_policy}}",
                            },
                        ],
                    },
                },
            },

        },
        {
            "name": "additional_paths",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+additional-paths\s(?P<value>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "additional-paths {{additional_paths}}",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "additional_paths": "{{value}}",
                    },
                },
            },
        },
        {
            "name": "as_path_loopcheck_out_disable",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+as-path-loopcheck\sout(?P<value>\sdisable)
                $""", re.VERBOSE,
            ),
            "setval": "as-path-loopcheck out disable",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "as_path_loopcheck_out_disable": "{{True if value is defined }}",
                    },
                },
            },
        },
        {
            "name": "bgp_attribute_download",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+bgp\s(?P<value>attribute-download)
                $""", re.VERBOSE,
            ),
            "setval": "bgp attribute-download",
            "compval": "bgp.attribute_download",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d()}}': {
                        "bgp": {
                            "attribute_download": "{{True if value is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_bestpath_origin_as_use",
            "getval": re.compile(
                r"""
                \s+bgp\sbestpath\s(?P<origin_as>origin-as\suse\svalidity)
                $""", re.VERBOSE,
            ),
            "setval": "bgp bestpath origin-as use validity",
            "compval": "bgp.bestpath.origin_as.use",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "bgp": {
                            "bestpath": {"origin_as": {"use": {"validity": "{{True if origin_as is defined }}"}}},
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_bestpath_origin_as_allow",
            "getval": re.compile(
                r"""
                \s+bgp\sbestpath\s(?P<origin_as>origin-as\sallow\sinvalid)
                $""", re.VERBOSE,
            ),
            "setval": "bgp bestpath origin-as allow invalid",
            "compval": "bgp.bestpath.origin_as.allow",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "bgp": {
                            "bestpath": {
                                "origin_as": {"allow": {"invalid": "{{True if origin_as is defined }}"}},
                            },
                        },
                    },
                },
            },

        },
        {
            "name": "bgp_reflection_disable",
            "getval": re.compile(
                r"""
                \s+bgp\sclient-to-client
                \sreflection
                \sdisable(?P<disable>)
                $""", re.VERBOSE,
            ),
            "setval": "bgp client-to-client reflection disable",
            "compval": "bgp.client_to_client.reflection.disable",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "bgp": {
                            "client_to_client": {
                                "reflection": {
                                    "disable": "{{True if disable is defined}}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_client_to_client_reflection_cluster_id",
            "getval": re.compile(
                r"""
                \s+bgp\sclient-to-client
                (\sreflection\scluster-id(?P<cs_id>\s\d+))?
                (\sdisable(?P<disable>))?
                $""", re.VERBOSE,
            ),
            "setval": "bgp client-to-clinet reflection cluster-id "
                      "{{ bgp.client_to_client.reflection.cluster_id }} disable",
            "compval": "bgp.client_to_client.reflection.cluster_id",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "bgp": {
                            "client_to_client": {
                                "reflection": {
                                    "cluster_id_disable": {
                                        "cluster_id": "{{cs_id}}",
                                        "disable": "{{True if disable is defined}}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_dampening",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+bgp\sdampening(?P<set>)
                (\s(?P<value>\d+))?
                (\sroute-policy\s(?P<route_policy>)\S+)?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_bgp_dampening,
            "compval": "bgp.dampening",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "bgp": {
                            "dampening": {
                                "set": "{{True if set is defined}}",
                                "value": "{{value}}",
                                "route_policy": "{{route_policy}}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_label_delay",
            "getval": re.compile(
                r"""
                \s+bgp\slabel-delay(?P<set>)
                (\s(?P<first>\S+))
                (\s(?P<second>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "bgp label-delay {{ bgp.label_delay.delay_second_parts}} {{ bgp.label_delay.delay_ms_parts}}",
            "compval": "bgp.label_delay",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "bgp": {
                            "label_delay": {
                                "delay_second_parts": "{{first}}",
                                "delay_ms_parts": "{{second}}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_import_delay",
            "getval": re.compile(
                r"""
                \s+bgp\simport-delay(?P<set>)
                (\s(?P<first>\S+))
                (\s(?P<second>\S+))
                $""", re.VERBOSE,
            ),
            "setval": "bgp import-delay {{ bgp.import_delay.delay_second_parts}} {{ bgp.import_delay.delay_ms_parts}}",
            "compval": "bgp.import_delay",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "bgp": {
                            "import_delay": {
                                "delay_second_parts": "{{first}}",
                                "delay_ms_parts": "{{second}}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_origin_as_validation",
            "getval": re.compile(
                r"""
                \s+bgp\sorigin-as\svalidation
                (\s(?P<disable>disable))?
                (\ssignal\s(?P<signal>ibgp))?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_bgp_origin_as_validation,
            "compval": "bgp.origin_as.validation",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "bgp": {
                            "origin_as": {
                                "validation": {
                                    "disable": "{{True if disable is defined}}",
                                    "signal": {
                                        "ibgp": "{{True if signal is defined}}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bgp_scan_time",
            "getval": re.compile(
                r"""
                \s+bgp\sscan-time\s(?P<scan_time>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "bgp scan-time {{bgp.scan_time}}",
            "compval": "bgp.scan_time",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "bgp": {
                            "scan_time": "{{scan_time}}",
                        },
                    },
                },
            },
        },
        {
            "name": "default_martian_check_disable",
            "getval": re.compile(
                r"""
                \s+default-martian-check(?P<disable>\sdisable)
                $""", re.VERBOSE,
            ),
            "setval": "default-martian-check disable",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "default_martian_check_disable": "{{ True if disable is defined}}",
                    },
                },
            },
        },
        {
            "name": "distance",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+distance\sbgp
                (\s(?P<external>\d+))?
                (\s(?P<internal>\d+))?
                (\s(?P<local>\d+))?
                $""", re.VERBOSE,
            ),
            "setval": "distnace bgp {{distnace.bgp.routes_external_to_as}} "
                      "{{distnace.bgp.routes_internal_to_as}} {{distnace.bgp.local_routes}}",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "distance": {
                            "routes_external_to_as": "{{external}}",
                            "routes_internal_to_as": "{{internal}}",
                            "local_routes": "{{local}}",
                        },
                    },
                },
            },
        },
        {
            "name": "dynamic_med",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+dynamic-med\sinterval\s(?P<dynamic_med>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "dynamic-med interval {{dynamic_med}}",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "dynamic_med": "{{ dynamic_med}}",
                    },
                },
            },
        },
        {
            "name": "maximum_paths_ibgp",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+maximum-paths\sibgp\s((?P<max_path_value>\S+))?
                (\sorder\sigp-metric(?P<order_igp_metric>))?
                (\sselective\sorder\sigp-metric(?P<selective_order_igp_metric>))?
                (\sunequal-cost(?P<unequal_cost>))?
                (\sorder\sigp-metric(?P<order_igp_metric1>))?
                (\sselective\sorder\sigp-metric(?P<selective_order_igp_metric1>))?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_maximum_paths_ibgp,
            "compval": "maximum_paths.ibgp",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "maximum_paths": {
                            "ibgp": {
                                "max_path_value": "{{ max_path_value }}",
                                "order_igp_metric": "{{ True if order_igp_metric is defined}}",
                                "selective_order_igp_metric":
                                "{{ True if selective_order_igp_metric is defined}}",
                                "unequal_cost": {
                                    "set": "{{ True if unequal_cost is defined}}",
                                    "order_igp_metric": "{{ True if order_igp_metric1 is defined}}",
                                    "selective_order_igp_metric": "{{ True if selective_order_igp_metric1 is defined}}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "maximum_paths_ebgp",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+maximum-paths\sebgp\s((?P<max_path_value>\S+))?
                (\sorder\sigp-metric(?P<order_igp_metric>))?
                (\sselective\sorder\sigp-metric(?P<selective_order_igp_metric>))?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_maximum_paths_ebgp,
            "compval": "maximum_paths.ebgp",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "maximum_paths": {
                            "ebgp": {
                                "max_path_value": "{{ max_path_value }}",
                                "order_igp_metric": "{{ True if order_igp_metric is defined}}",
                                "selective_order_igp_metric":
                                "{{ True if selective_order_igp_metric is defined}}",

                            },
                        },
                    },
                },
            },
        },
        {
            "name": "maximum_paths_eibgp",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+maximum-paths\seibgp\s((?P<max_path_value>\S+))?
                (\sorder\sigp-metric(?P<order_igp_metric>))?
                (\sselective\sorder\sigp-metric(?P<selective_order_igp_metric>))?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_maximum_paths_eibgp,
            "compval": "maximum_paths.eibgp",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "maximum_paths": {
                            "eibgp": {
                                "max_path_value": "{{ max_path_value }}",
                                "order_igp_metric": "{{ True if order_igp_metric is defined}}",
                                "selective_order_igp_metric":
                                "{{ True if selective_order_igp_metric is defined}}",

                            },
                        },
                    },
                },
            },
        },
        {
            "name": "networks",
            "getval": re.compile(
                r"""
                \s+network\s(?P<value>\S+)
                (\sbackdoor-route-policy\s(?P<backdoor_route_policy>)\S+)?
                (\sroute-policy\s(?P<route_policy>)\S+)?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_network,
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "networks": [
                            {
                                "backdoor_route_policy": "{{backdoor_route_policy}}",
                                "network": "{{value}}",
                                "route_policy": "{{route_policy}}",
                            },
                        ],
                    },
                },
            },
        },
        {
            "name": "nexthop",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+nexthop
                (\sresolution\sprefix-length\sminimum\s(?P<value>\d+))?
                (\strigger-delay\scritical\s(?P<trigger_delay_critical>\d+))?
                 (\strigger-delay\snon-critical\s(?P<trigger_delay_non_critical>\d+))?
                (\sroute-policy\s(?P<route_policy>)\S+)?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_nexthop,
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "nexthop": {
                            "trigger_delay_critical": "{{trigger_delay_critical}}",
                            "trigger_delay_non_critical": "{{trigger_delay_non_critical}}",
                            "resolution_prefix_length_minimum": "{{value}}",
                            "route_policy": "{{route_policy}}",
                        },
                    },
                },
            },
        },
        {
            "name": "optimal_route_reflection",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+optimal-route-reflection
                (\s(?P<value>\S+))?
                (\s(?P<primary>\S+))?
                (\s(?P<secondary>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_optimal_route,
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "optimal_route_reflection": {
                            "group_name": "{{value}}",
                            "primary_address": "{{primary}}",
                            "secondary_address": "{{secondary}}",
                        },
                    },
                },
            },
        },
        {
            "name": "permanent_network_route_policy",
            "getval": re.compile(
                r"""
                \s+permanent-network\s(?P<value>route_policy\S+)?
                $""", re.VERBOSE,
            ),
            "setval": "permanent-network route-policy {{permanent_network_route_policy}}",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "permanent_network_route_policy": "{{route_policy}}",
                    },
                },
            },
        },
        {
            "name": "retain_local_label",
            "getval": re.compile(
                r"""
                \s+retain\slocal-label\s(?P<value>\d+)?
                $""", re.VERBOSE,
            ),
            "setval": "retain local-label {{retain_local_label}}",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "retain_local_label": "{{value}}",
                    },
                },
            },
        },
        {
            "name": "update",
            "getval": re.compile(
                r"""
                \s+update
                (\slimit)?
                (\ssub-group)?
                (\sibgp\s(?P<ibgp>\d+))?
                 (\sebgp\s(?P<ebgp>\d+))?
                (\saddress-family\s(?P<af>\d+))?
                (\swait-install(?P<wait>))?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_update,
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "update": {
                            "wait_install": "{{True if wait is defined}}",
                            "limit": {
                                "sub_group": {
                                    "ibgp": "{{ibgp}}",
                                    "ebgp": "{{ebgp}}",
                                },
                                "address_family": "{{af}}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "redistribute",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+redistribute
                \s(?P<protocol>\S+)
                (\s(?P<id>\S+))?
                (\smetric\s(?P<metric>\d+))?
                (\slevel\s(?P<level>\S+))?
                (\sinternal(?P<internal>))?
                (\sexternal(?P<external>)(\s(?P<ospf_external>\S+))?)?
                (\snssa-external(?P<nssa_external>))?
                (\sroute-policy\s(?P<route_policy>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": _tmplt_redistribute,
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "redistribute": [
                            {
                                "protocol": "{{protocol}}",
                                "id": "{{id}}",
                                "route_policy": "{{route_policy}}",
                                "metric": "{{metric}}",
                                "internal": "{{True if internal is defined}}",
                                "external": "{{True if external is defined}}",
                                "level": "{{level}}",
                                "ospf_external": "{{ospf_external}}",
                                "nssa_external": "{{True if nssa_external is defined}}",
                            },
                        ],
                    },
                },
            },
        },
        {
            "name": "inter_as_install",
            "getval": re.compile(
                r"""
                \s+inter-as\s(?P<inter_as>install)
                $""", re.VERBOSE,
            ),
            "setval": "inter-as install",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "inter_as_install": "{{True if inter_as is defined}}",
                    },
                },
            },

        },
        {
            "name": "segmented_multicast",
            "getval": re.compile(
                r"""
                \s+(?P<segmented_multicast>segmented-multicast)
                $""", re.VERBOSE,
            ),
            "setval": "segmented-multicast",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "segmented_multicast": "{{True if segmented_multicast is defined}}",
                    },
                },
            },

        },
        {
            "name": "global_table_multicast",
            "getval": re.compile(
                r"""
                \s+(?P<global_table_multicast>global-table-multicast)
                $""", re.VERBOSE,
            ),
            "setval": "global-table-multicast",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "global_table_multicast": "{{True if global_table_multicast is defined}}",
                    },
                },
            },

        },
        {
            "name": "table_policy",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                (\s+table-policy\s(?P<table_policy>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "table-policy {{table_policy}}",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "table_policy": "{{table_policy}}",
                    },
                },
            },
        },
        {
            "name": "vrf_all_conf",
            "getval": re.compile(
                r"""
                \s+vrf\sall
                (\s+source\srt(?P<source_rt_import_policy>\simport-policy))?
                (\s+table-policy\s(?P<table_policy>\S+))?
                (\s+label\smode)?
                (\s+(?P<per_ce>per-ce))?
                (\s+(?P<per_vrf>per-vrf))?
                (\s+route_policy\s(?P<route_policy>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_vrf_all,
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "vrf_all_conf": {
                            "source_rt_import_policy": "{{ True if source_rt_import_policy is defined}}",
                            "table_policy": "{{table_policy}}",
                            "label_mode": {
                                "per_ce": "{{ True if per_ce is defined}}",
                                "per_vrf": "{{True if per_vrf is defined}}",
                                "route_policy": "{{route_policy}}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "weight",
            "getval": re.compile(
                r"""
                \s+weight
                (\sreset-on-import\sdisable(?P<reset_on_import_disable>))?
                (\sreset-on-import(?P<reset_on_import>))?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_wt,
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "weight": {
                            "reset_on_import_disable": "{{ True if reset_on_import_disable is defined}}",
                            "reset_on_import": "{{ True if reset_on_import is defined}}",
                        },
                    },
                },
            },
        },
        {
            "name": "route_target_download",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+route-target\s(?P<value>download)
                $""", re.VERBOSE,
            ),
            "setval": "route-target download",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "route_target_download": "{{True if value is defined }}",

                    },
                },
            },
        },
        {
            "name": "mvpn_single_forwarder_selection_all",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+mvpn\ssingle-forwarder-selection\s(?P<value>all)
                $""", re.VERBOSE,
            ),
            "setval": "mvpn single-forwarder-selection all",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "mvpn_single_forwarder_selection_all": "{{True if value is defined }}",
                    },
                },
            },
        },
        {
            "name": "mvpn_single_forwarder_selection_highest_ip_address",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+mvpn\ssingle-forwarder-selection\s(?P<value>highest-ip-address)
                $""", re.VERBOSE,
            ),
            "setval": "mvpn single-forwarder-selection highest-ip-address",
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "mvpn_single_forwarder_selection_highest_ip_address": "{{True if value is defined }}",
                    },
                },
            },
        },
        {
            "name": "label_mode",
            "getval": re.compile(
                r"""
                (\s+vrf\s(?P<vrf>\S+))?
                \s+label\smode\s(?P<per_ce>per-ce)?(?P<per_prefix>per-prefix)?
                (?P<per_vrf>per-vrf)?
                (?P<rr>route-policy\s\S+)?
                $""", re.VERBOSE,
            ),
            "setval": _tmpl_label_mode,
            "result": {
                "address_family": {
                    '{{"address_family_" + afi + "_" + safi + "_vrf_" + vrf|d() }}': {
                        "label_mode": {
                            "per_ce": "{{ True if per_ce is defined}}",
                            "per_prefix": "{{ True if per_prefix is defined}}",
                            "per_vrf": "{{ True if per_vrf is defined}}",
                            "route_policy": "{{ route_policy}}",
                        },
                    },
                },
            },
        },
    ]
    # fmt: on



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_system
extends_documentation_fragment:
- cisco.nxos.nxos
author: Peter Sprygada (@privateip)
short_description: Manage the system attributes on Cisco NXOS devices
notes:
- Unsupported for Cisco MDS
description:
- This module provides declarative management of node system attributes on Cisco NXOS
  devices.  It provides an option to configure host system parameters or remove those
  parameters from the device active configuration.
version_added: 1.0.0
options:
  hostname:
    description:
    - Configure the device hostname parameter. This option takes an ASCII string value
      or keyword 'default'
    type: str
  domain_name:
    description:
    - Configures the default domain name suffix to be used when referencing this node
      by its FQDN.  This argument accepts either a list of domain names or a list
      of dicts that configure the domain name and VRF name or keyword 'default'. See
      examples.
    type: list
    elements: raw
  domain_lookup:
    description:
    - Enables or disables the DNS lookup feature in Cisco NXOS.  This argument accepts
      boolean values.  When enabled, the system will try to resolve hostnames using
      DNS and when disabled, hostnames will not be resolved.
    type: bool
  domain_search:
    description:
    - Configures a list of domain name suffixes to search when performing DNS name
      resolution. This argument accepts either a list of domain names or a list of
      dicts that configure the domain name and VRF name or keyword 'default'. See
      examples.
    type: list
    elements: raw
  name_servers:
    description:
    - List of DNS name servers by IP address to use to perform name resolution lookups.  This
      argument accepts either a list of DNS servers or a list of hashes that configure
      the name server and VRF name or keyword 'default'. See examples.
    type: list
    elements: raw
  system_mtu:
    description:
    - Specifies the mtu, must be an integer or keyword 'default'.
    type: str
  state:
    description:
    - State of the configuration values in the device's current active configuration.  When
      set to I(present), the values should be configured in the device active configuration
      and when set to I(absent) the values should not be in the device active configuration
    default: present
    choices:
    - present
    - absent
    type: str
"""

EXAMPLES = """
- name: configure hostname and domain-name
  cisco.nxos.nxos_system:
    hostname: nxos01
    domain_name: test.example.com

- name: remove configuration
  cisco.nxos.nxos_system:
    state: absent

- name: configure name servers
  cisco.nxos.nxos_system:
    name_servers:
    - 8.8.8.8
    - 8.8.4.4

- name: configure name servers with VRF support
  cisco.nxos.nxos_system:
    name_servers:
    - {server: 8.8.8.8, vrf: mgmt}
    - {server: 8.8.4.4, vrf: mgmt}
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always
  type: list
  sample:
    - hostname nxos01
    - ip domain-name test.example.com
"""
import re

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.config import (
    NetworkConfig,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    ComplexList,
)

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    get_config,
    load_config,
)


_CONFIGURED_VRFS = None


def has_vrf(module, vrf):
    global _CONFIGURED_VRFS
    if _CONFIGURED_VRFS is not None:
        return vrf in _CONFIGURED_VRFS
    config = get_config(module)
    _CONFIGURED_VRFS = re.findall(r"vrf context (\S+)", config)
    return vrf in _CONFIGURED_VRFS


def map_obj_to_commands(want, have, module):
    commands = list()
    state = module.params["state"]

    def needs_update(x):
        return want.get(x) and (want.get(x) != have.get(x))

    def difference(x, y, z):
        return [item for item in x[z] if item not in y[z]]

    def remove(cmd, commands, vrf=None):
        if vrf:
            commands.append("vrf context %s" % vrf)
        commands.append(cmd)
        if vrf:
            commands.append("exit")

    def add(cmd, commands, vrf=None):
        if vrf:
            if not has_vrf(module, vrf):
                module.fail_json(msg="invalid vrf name %s" % vrf)
        return remove(cmd, commands, vrf)

    if state == "absent":
        if have["hostname"]:
            commands.append("no hostname")

        for item in have["domain_name"]:
            cmd = "no ip domain-name %s" % item["name"]
            remove(cmd, commands, item["vrf"])

        for item in have["domain_search"]:
            cmd = "no ip domain-list %s" % item["name"]
            remove(cmd, commands, item["vrf"])

        for item in have["name_servers"]:
            cmd = "no ip name-server %s" % item["server"]
            remove(cmd, commands, item["vrf"])

        if have["system_mtu"]:
            commands.append("no system jumbomtu")

    if state == "present":
        if needs_update("hostname"):
            if want["hostname"] == "default":
                if have["hostname"]:
                    commands.append("no hostname")
            else:
                commands.append("hostname %s" % want["hostname"])

        if want.get("domain_lookup") is not None:
            if have.get("domain_lookup") != want.get("domain_lookup"):
                cmd = "ip domain-lookup"
                if want["domain_lookup"] is False:
                    cmd = "no %s" % cmd
                commands.append(cmd)

        if want["domain_name"]:
            if want.get("domain_name")[0]["name"] == "default":
                if have["domain_name"]:
                    for item in have["domain_name"]:
                        cmd = "no ip domain-name %s" % item["name"]
                        remove(cmd, commands, item["vrf"])
            else:
                for item in difference(have, want, "domain_name"):
                    cmd = "no ip domain-name %s" % item["name"]
                    remove(cmd, commands, item["vrf"])
                for item in difference(want, have, "domain_name"):
                    cmd = "ip domain-name %s" % item["name"]
                    add(cmd, commands, item["vrf"])

        if want["domain_search"]:
            if want.get("domain_search")[0]["name"] == "default":
                if have["domain_search"]:
                    for item in have["domain_search"]:
                        cmd = "no ip domain-list %s" % item["name"]
                        remove(cmd, commands, item["vrf"])
            else:
                for item in difference(have, want, "domain_search"):
                    cmd = "no ip domain-list %s" % item["name"]
                    remove(cmd, commands, item["vrf"])
                for item in difference(want, have, "domain_search"):
                    cmd = "ip domain-list %s" % item["name"]
                    add(cmd, commands, item["vrf"])

        if want["name_servers"]:
            if want.get("name_servers")[0]["server"] == "default":
                if have["name_servers"]:
                    for item in have["name_servers"]:
                        cmd = "no ip name-server %s" % item["server"]
                        remove(cmd, commands, item["vrf"])
            else:
                for item in difference(have, want, "name_servers"):
                    cmd = "no ip name-server %s" % item["server"]
                    remove(cmd, commands, item["vrf"])
                for item in difference(want, have, "name_servers"):
                    cmd = "ip name-server %s" % item["server"]
                    add(cmd, commands, item["vrf"])

        if needs_update("system_mtu"):
            if want["system_mtu"] == "default":
                if have["system_mtu"]:
                    commands.append("no system jumbomtu")
            else:
                commands.append("system jumbomtu %s" % want["system_mtu"])

    return commands


def parse_hostname(config):
    match = re.search(r"^hostname (\S+)", config, re.M)
    if match:
        return match.group(1)


def parse_domain_name(config, vrf_config):
    objects = list()
    match = re.search(r"^ip domain-name (\S+)", config, re.M)
    if match:
        objects.append({"name": match.group(1), "vrf": None})

    for vrf, cfg in iteritems(vrf_config):
        match = re.search(r"ip domain-name (\S+)", cfg, re.M)
        if match:
            objects.append({"name": match.group(1), "vrf": vrf})

    return objects


def parse_domain_search(config, vrf_config):
    objects = list()

    for item in re.findall(r"^ip domain-list (\S+)", config, re.M):
        objects.append({"name": item, "vrf": None})

    for vrf, cfg in iteritems(vrf_config):
        for item in re.findall(r"ip domain-list (\S+)", cfg, re.M):
            objects.append({"name": item, "vrf": vrf})

    return objects


def parse_name_servers(config, vrf_config, vrfs):
    objects = list()

    match = re.search("^ip name-server (.+)$", config, re.M)
    if match and "use-vrf" not in match.group(1):
        for addr in match.group(1).split(" "):
            objects.append({"server": addr, "vrf": None})

    for vrf, cfg in iteritems(vrf_config):
        vrf_match = re.search("ip name-server (.+)", cfg, re.M)
        if vrf_match:
            for addr in vrf_match.group(1).split(" "):
                objects.append({"server": addr, "vrf": vrf})

    return objects


def parse_system_mtu(config):
    match = re.search(r"^system jumbomtu (\d+)", config, re.M)
    if match:
        return match.group(1)


def map_config_to_obj(module):
    config = get_config(module)
    configobj = NetworkConfig(indent=2, contents=config)

    vrf_config = {}

    vrfs = re.findall(r"^vrf context (\S+)$", config, re.M)
    for vrf in vrfs:
        config_data = configobj.get_block_config(path=["vrf context %s" % vrf])
        vrf_config[vrf] = config_data

    return {
        "hostname": parse_hostname(config),
        "domain_lookup": "no ip domain-lookup" not in config,
        "domain_name": parse_domain_name(config, vrf_config),
        "domain_search": parse_domain_search(config, vrf_config),
        "name_servers": parse_name_servers(config, vrf_config, vrfs),
        "system_mtu": parse_system_mtu(config),
    }


def map_params_to_obj(module):
    obj = {
        "hostname": module.params["hostname"],
        "domain_lookup": module.params["domain_lookup"],
        "system_mtu": module.params["system_mtu"],
    }

    domain_name = ComplexList(dict(name=dict(key=True), vrf=dict()), module)

    domain_search = ComplexList(dict(name=dict(key=True), vrf=dict()), module)

    name_servers = ComplexList(dict(server=dict(key=True), vrf=dict()), module)

    for arg, cast in [
        ("domain_name", domain_name),
        ("domain_search", domain_search),
        ("name_servers", name_servers),
    ]:
        if module.params[arg] is not None:
            obj[arg] = cast(module.params[arg])
        else:
            obj[arg] = None

    return obj


def main():
    """main entry point for module execution"""
    argument_spec = dict(
        hostname=dict(),
        domain_lookup=dict(type="bool"),
        # { name: <str>, vrf: <str> }
        domain_name=dict(type="list", elements="raw"),
        # {name: <str>, vrf: <str> }
        domain_search=dict(type="list", elements="raw"),
        # { server: <str>; vrf: <str> }
        name_servers=dict(type="list", elements="raw"),
        system_mtu=dict(type="str"),
        state=dict(default="present", choices=["present", "absent"]),
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    warnings = list()

    result = {"changed": False}
    if warnings:
        result["warnings"] = warnings

    want = map_params_to_obj(module)
    have = map_config_to_obj(module)

    commands = map_obj_to_commands(want, have, module)
    result["commands"] = commands

    if commands:
        if not module.check_mode:
            load_config(module, commands)
        result["changed"] = True

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_switch_qos_rules_order
short_description: Resource module for networks _switch _qosrules _order
description:
- Manage operations create, update and delete of the resource networks _switch _qosrules _order.
- Add a quality of service rule.
- Delete a quality of service rule.
- Update a quality of service rule.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  dscp:
    description: DSCP tag. Set this to -1 to trust incoming DSCP. Default value is 0.
    type: int
  dstPort:
    description: The destination port of the incoming packet. Applicable only if protocol
      is TCP or UDP.
    type: int
  dstPortRange:
    description: The destination port range of the incoming packet. Applicable only
      if protocol is set to TCP or UDP. Example 70-80.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  protocol:
    description: The protocol of the incoming packet. Can be one of "ANY", "TCP" or
      "UDP". Default value is "ANY".
    type: str
  qosRuleId:
    description: QosRuleId path parameter. Qos rule ID.
    type: str
  srcPort:
    description: The source port of the incoming packet. Applicable only if protocol
      is TCP or UDP.
    type: int
  srcPortRange:
    description: The source port range of the incoming packet. Applicable only if protocol
      is set to TCP or UDP. Example 70-80.
    type: str
  vlan:
    description: The VLAN of the incoming packet. A null value will match any VLAN.
    type: int
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for switch createNetworkSwitchQosRule
  description: Complete reference of the createNetworkSwitchQosRule API.
  link: https://developer.cisco.com/meraki/api-v1/#!create-network-switch-qos-rule
- name: Cisco Meraki documentation for switch deleteNetworkSwitchQosRule
  description: Complete reference of the deleteNetworkSwitchQosRule API.
  link: https://developer.cisco.com/meraki/api-v1/#!delete-network-switch-qos-rule
- name: Cisco Meraki documentation for switch updateNetworkSwitchQosRule
  description: Complete reference of the updateNetworkSwitchQosRule API.
  link: https://developer.cisco.com/meraki/api-v1/#!update-network-switch-qos-rule
notes:
  - SDK Method used are
    switch.Switch.create_network_switch_qos_rule,
    switch.Switch.delete_network_switch_qos_rule,
    switch.Switch.update_network_switch_qos_rule,

  - Paths used are
    post /networks/{networkId}/switch/qosRules,
    delete /networks/{networkId}/switch/qosRules/{qosRuleId},
    put /networks/{networkId}/switch/qosRules/{qosRuleId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_switch_qos_rules_order:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    dscp: 0
    dstPort: 3000
    dstPortRange: 3000-3100
    networkId: string
    protocol: TCP
    srcPort: 2000
    srcPortRange: 70-80
    vlan: 100

- name: Update by id
  cisco.meraki.networks_switch_qos_rules_order:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: present
    dscp: 0
    dstPort: 3000
    dstPortRange: 3000-3100
    networkId: string
    protocol: TCP
    qosRuleId: string
    srcPort: 2000
    srcPortRange: 70-80
    vlan: 100

- name: Delete by id
  cisco.meraki.networks_switch_qos_rules_order:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    state: absent
    networkId: string
    qosRuleId: string

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: site_design_floormap_info
short_description: Information module for Site Design Floormap
description:
- Get all Site Design Floormap.
- Get Site Design Floormap by id.
- List all floor maps.
- List specified floor map(s).
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  floorId:
    description:
    - FloorId path parameter. Group Id of the specified floormap.
    type: str
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
notes:
  - SDK Method used are
    site_design.SiteDesign.get_floormap,
    site_design.SiteDesign.get_floormaps,

  - Paths used are
    get /dna/intent/api/v1/wireless/floormap/all,
    get /dna/intent/api/v1/wireless/floormap/{floorId},

"""

EXAMPLES = r"""
- name: Get all Site Design Floormap
  cisco.dnac.site_design_floormap_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers:
      custom: value
  register: result

- name: Get Site Design Floormap by id
  cisco.dnac.site_design_floormap_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers:
      custom: value
    floorId: string
  register: result

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample:
  - {}
"""



# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The asa legacy fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


import platform
import re

from ansible_collections.cisco.asa.plugins.module_utils.network.asa.asa import (
    get_capabilities,
    run_commands,
)


class FactsBase(object):
    COMMANDS = list()

    def __init__(self, module):
        self.module = module
        self.facts = dict()
        self.warnings = list()
        self.responses = None

    def populate(self):
        self.responses = run_commands(
            self.module,
            commands=self.COMMANDS,
            check_rc=False,
        )

    def run(self, cmd):
        return run_commands(self.module, commands=cmd, check_rc=False)


class Default(FactsBase):
    COMMANDS = ["show version"]

    def populate(self):
        super(Default, self).populate()
        self.facts.update(self.platform_facts())
        data = self.responses[0]
        if data:
            self.facts["asatype"] = self.parse_asatype(data)
            self.facts["serialnum"] = self.parse_serialnum(data)
            self.parse_stacks(data)

    def parse_asatype(self, data):
        match = re.search(r"Hardware:(\s+)ASA", data)
        if match:
            return "ASA"

    def parse_serialnum(self, data):
        match = re.search(r"Serial Number: (\S+)", data)
        if match:
            return match.group(1)

    def parse_stacks(self, data):
        match = re.findall(r"^Model [Nn]umber\s+: (\S+)", data, re.M)
        if match:
            self.facts["stacked_models"] = match

        match = re.findall(
            r"^System [Ss]erial [Nn]umber\s+: (\S+)",
            data,
            re.M,
        )
        if match:
            self.facts["stacked_serialnums"] = match

    def platform_facts(self):
        platform_facts = {}

        resp = get_capabilities(self.module)
        device_info = resp["device_info"]
        platform_facts["system"] = device_info["network_os"]

        for item in (
            "model",
            "image",
            "version",
            "platform",
            "hostname",
            "firepower_version",
            "device_mgr_version",
        ):
            val = device_info.get("network_os_%s" % item)
            if val:
                platform_facts[item] = val

        platform_facts["api"] = resp["network_api"]
        platform_facts["python_version"] = platform.python_version()

        return platform_facts


class Hardware(FactsBase):
    COMMANDS = ["dir", "show memory"]

    def populate(self):
        warnings = list()
        super(Hardware, self).populate()
        data = self.responses[0]
        if data:
            self.facts["filesystems"] = self.parse_filesystems(data)
            self.facts["filesystems_info"] = self.parse_filesystems_info(data)

        data = self.responses[1]
        if data:
            if "Invalid input detected" in data:
                warnings.append("Unable to gather memory statistics")
            else:
                mem_list = [l for l in data.splitlines() if "memory" in l]
                for each in mem_list:
                    if "Free memory" in each:
                        match = re.search(
                            r"Free memory.+ (\d+) .+(\d\d)",
                            each,
                        )
                        if match:
                            self.facts["memfree_mb"] = int(match.group(1)) // 1024
                    elif "Used memory" in each:
                        match = re.search(
                            r"Used memory.+ (\d+) .+(\d\d)",
                            each,
                        )
                        if match:
                            self.facts["memused_mb"] = int(match.group(1)) // 1024
                    elif "Total memory" in each:
                        match = re.search(
                            r"Total memory.+ (\d+) .+(\d\d)",
                            each,
                        )
                        if match:
                            self.facts["memtotal_mb"] = int(match.group(1)) // 1024

    def parse_filesystems(self, data):
        return re.findall(r"^Directory of (\S+)/", data, re.M)

    def parse_filesystems_info(self, data):
        facts = dict()
        fs = ""
        for line in data.split("\n"):
            match = re.match(r"^Directory of (\S+)/", line)
            if match:
                fs = match.group(1)
                facts[fs] = dict()
                continue
            match = re.match(
                r"^(\d+) bytes total \((\d+) bytes free\/(\d+)% free\)",
                line,
            )
            if match:
                facts[fs]["spacetotal_kb"] = int(match.group(1)) / 1024
                facts[fs]["spacefree_kb"] = int(match.group(2)) / 1024

        return facts


class Config(FactsBase):
    COMMANDS = ["show running-config"]

    def populate(self):
        super(Config, self).populate()
        data = self.responses[0]
        if data:
            data = re.sub(
                r"^Building configuration...\s+Current configuration : \d+ bytes\n",
                "",
                data,
                flags=re.MULTILINE,
            )
            self.facts["config"] = data



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2017, Bruno Calogero <brunocalogero@hotmail.com>
# Copyright: (c) 2023, Anvitha Jain <anvjain@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_interface_policy_leaf_policy_group
short_description: Manage fabric interface policy leaf policy groups (infra:AccBndlGrp, infra:AccPortGrp)
description:
- Manage fabric interface policy leaf policy groups on Cisco ACI fabrics.
options:
  policy_group:
    description:
    - The name of the leaf interface policy group.
    type: str
    aliases: [ name, policy_group_name ]
  description:
    description:
    - The description of the leaf interface policy group.
    type: str
    aliases: [ descr ]
  lag_type:
    description:
    - Selector for the type of leaf interface policy group.
    - C(leaf) for Leaf Access Port Policy Group
    - C(link) for Port Channel (PC)
    - C(node) for Virtual Port Channel (VPC)
    type: str
    required: true
    choices: [ leaf, link, node ]
    aliases: [ lag_type_name ]
  link_level_policy:
    description:
    - The name of the link level policy used by the leaf interface policy group.
    type: str
    aliases: [ link_level_policy_name ]
  cdp_policy:
    description:
    - The name of the cdp policy used by the leaf interface policy group.
    type: str
    aliases: [ cdp_policy_name ]
  mcp_policy:
    description:
    - The name of the mcp policy used by the leaf interface policy group.
    type: str
    aliases: [ mcp_policy_name ]
  lldp_policy:
    description:
    - The name of the lldp policy used by the leaf interface policy group.
    type: str
    aliases: [ lldp_policy_name ]
  stp_interface_policy:
    description:
    - The name of the stp interface policy used by the leaf interface policy group.
    type: str
    aliases: [ stp_interface_policy_name ]
  egress_data_plane_policing_policy:
    description:
    - The name of the egress data plane policing policy used by the leaf interface policy group.
    type: str
    aliases: [ egress_data_plane_policing_policy_name ]
  ingress_data_plane_policing_policy:
    description:
    - The name of the ingress data plane policing policy used by the leaf interface policy group.
    type: str
    aliases: [ ingress_data_plane_policing_policy_name ]
  priority_flow_control_policy:
    description:
    - The name of the priority flow control policy used by the leaf interface policy group.
    type: str
    aliases: [ priority_flow_control_policy_name ]
  fibre_channel_interface_policy:
    description:
    - The name of the fibre channel interface policy used by the leaf interface policy group.
    type: str
    aliases: [ fibre_channel_interface_policy_name ]
  slow_drain_policy:
    description:
    - The name of the slow drain policy used by the leaf interface policy group.
    type: str
    aliases: [ slow_drain_policy_name ]
  port_channel_policy:
    description:
    - The name of the port channel policy used by the leaf interface policy group.
    type: str
    aliases: [ port_channel_policy_name ]
  monitoring_policy:
    description:
    - The name of the monitoring policy used by the leaf interface policy group.
    type: str
    aliases: [ monitoring_policy_name ]
  storm_control_interface_policy:
    description:
    - The name of the storm control interface policy used by the leaf interface policy group.
    type: str
    aliases: [ storm_control_interface_policy_name ]
  l2_interface_policy:
    description:
    - The name of the l2 interface policy used by the leaf interface policy group.
    type: str
    aliases: [ l2_interface_policy_name ]
  port_security_policy:
    description:
    - The name of the port security policy used by the leaf interface policy group.
    type: str
    aliases: [ port_security_policy_name ]
  link_flap_policy:
    description:
    - The name of the link flap policy used by the leaf interface policy group.
    type: str
    aliases: [ link_flap_policy_name ]
  link_level_flow_control:
    description:
    - The name of the link level flow control used by the leaf interface policy group.
    type: str
    aliases: [ link_level_flow_control_name ]
  mac_sec_interface_policy:
    description:
    - The name of the mac sec interface policy used by the leaf interface policy group.
    type: str
    aliases: [ mac_sec_interface_policy_name ]
  copp_policy:
    description:
    - The name of the copp policy used by the leaf interface policy group.
    type: str
    aliases: [ copp_policy_name ]
  sync_e_interface_policy:
    description:
    - The name of the syncE interface policy used by the leaf interface policy group.
    - Only availavle in APIC version 5.2 or later.
    type: str
    aliases: [ sync_e_interface_policy_name ]
  port_authentication:
    description:
    - The name of the port authentication used by the leaf interface policy group.
    type: str
    aliases: [ port_authentication_name ]
  dwdm:
    description:
    - The name of the dwdm used by the leaf interface policy group.
    type: str
    aliases: [ dwdm_name ]
  poe_interface_policy:
    description:
    - The name of the poe interface policy used by the leaf interface policy group.
    type: str
    aliases: [ poe_interface_policy_name ]
  transceiver_policy:
    description:
    - The name of the transceiver policy used by the leaf interface policy group.
    - Only availavle in APIC version 6.0(2h) or later.
    type: dict
    suboptions:
      type:
        description:
        - The type of the transceiver policy.
        type: str
        required: true
        aliases: [ transceiver_policy_type ]
        choices: [ zr, zrp]
      name:
        description:
        - The name of the transceiver policy.
        type: str
        required: true
        aliases: [ transceiver_policy_name ]
  aep:
    description:
    - The name of the attached entity profile (AEP) used by the leaf interface policy group.
    type: str
    aliases: [ aep_name ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- When using the module please select the appropriate link_aggregation_type (lag_type).
- C(link) for Port Channel(PC), C(node) for Virtual Port Channel(VPC) and C(leaf) for Leaf Access Port Policy Group.
seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes B(infra:AccBndlGrp) and B(infra:AccPortGrp).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Bruno Calogero (@brunocalogero)
"""

EXAMPLES = r"""
- name: Create a Port Channel (PC) Interface Policy Group
  cisco.aci.aci_interface_policy_leaf_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    lag_type: link
    policy_group: policygroupname
    description: policygroupname description
    link_level_policy: linklevelpolicy
    cdp_policy: cdppolicy
    lldp_policy: lldppolicy
    port_channel_policy: lacppolicy
    state: present
  delegate_to: localhost

- name: Create a Virtual Port Channel (VPC) Interface Policy Group
  cisco.aci.aci_interface_policy_leaf_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    lag_type: node
    policy_group: policygroupname
    link_level_policy: linklevelpolicy
    cdp_policy: cdppolicy
    lldp_policy: lldppolicy
    port_channel_policy: lacppolicy
    state: present
  delegate_to: localhost

- name: Create a Leaf Access Port Policy Group
  cisco.aci.aci_interface_policy_leaf_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    lag_type: leaf
    policy_group: policygroupname
    link_level_policy: linklevelpolicy
    cdp_policy: cdppolicy
    lldp_policy: lldppolicy
    state: present
  delegate_to: localhost

- name: Query all Leaf Access Port Policy Groups of type link
  cisco.aci.aci_interface_policy_leaf_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    lag_type: link
    state: query
  delegate_to: localhost
  register: query_result

- name: Query a specific Lead Access Port Policy Group
  cisco.aci.aci_interface_policy_leaf_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    lag_type: leaf
    policy_group: policygroupname
    state: query
  delegate_to: localhost
  register: query_result

- name: Delete an Interface policy Leaf Policy Group
  cisco.aci.aci_interface_policy_leaf_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    lag_type: leaf
    policy_group: policygroupname
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        # NOTE: Since this module needs to include both infra:AccBndlGrp (for PC and VPC) and infra:AccPortGrp (for leaf access port policy group):
        # NOTE: The user(s) can make the choice between (link(PC), node(VPC), leaf(leaf-access port policy group))
        lag_type=dict(type="str", required=True, aliases=["lag_type_name"], choices=["leaf", "link", "node"]),
        policy_group=dict(type="str", aliases=["name", "policy_group_name"]),  # Not required for querying all objects
        description=dict(type="str", aliases=["descr"]),
        link_level_policy=dict(type="str", aliases=["link_level_policy_name"]),
        cdp_policy=dict(type="str", aliases=["cdp_policy_name"]),
        mcp_policy=dict(type="str", aliases=["mcp_policy_name"]),
        lldp_policy=dict(type="str", aliases=["lldp_policy_name"]),
        stp_interface_policy=dict(type="str", aliases=["stp_interface_policy_name"]),
        egress_data_plane_policing_policy=dict(type="str", aliases=["egress_data_plane_policing_policy_name"]),
        ingress_data_plane_policing_policy=dict(type="str", aliases=["ingress_data_plane_policing_policy_name"]),
        priority_flow_control_policy=dict(type="str", aliases=["priority_flow_control_policy_name"]),
        fibre_channel_interface_policy=dict(type="str", aliases=["fibre_channel_interface_policy_name"]),
        slow_drain_policy=dict(type="str", aliases=["slow_drain_policy_name"]),
        port_channel_policy=dict(type="str", aliases=["port_channel_policy_name"]),
        monitoring_policy=dict(type="str", aliases=["monitoring_policy_name"]),
        storm_control_interface_policy=dict(type="str", aliases=["storm_control_interface_policy_name"]),
        l2_interface_policy=dict(type="str", aliases=["l2_interface_policy_name"]),
        port_security_policy=dict(type="str", aliases=["port_security_policy_name"]),
        link_flap_policy=dict(type="str", aliases=["link_flap_policy_name"]),
        link_level_flow_control=dict(type="str", aliases=["link_level_flow_control_name"]),
        mac_sec_interface_policy=dict(type="str", aliases=["mac_sec_interface_policy_name"]),
        copp_policy=dict(type="str", aliases=["copp_policy_name"]),
        aep=dict(type="str", aliases=["aep_name"]),
        sync_e_interface_policy=dict(type="str", aliases=["sync_e_interface_policy_name"]),
        transceiver_policy=dict(
            type="dict",
            options=dict(
                name=dict(type="str", required=True, aliases=["transceiver_policy_name"]),
                type=dict(type="str", required=True, choices=["zr", "zrp"], aliases=["transceiver_policy_type"]),
            ),
        ),
        poe_interface_policy=dict(type="str", aliases=["poe_interface_policy_name"]),
        port_authentication=dict(type="str", aliases=["port_authentication_name"]),
        dwdm=dict(type="str", aliases=["dwdm_name"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        name_alias=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["policy_group"]],
            ["state", "present", ["policy_group"]],
        ],
    )

    policy_group = module.params.get("policy_group")
    description = module.params.get("description")
    lag_type = module.params.get("lag_type")
    link_level_policy = module.params.get("link_level_policy")
    cdp_policy = module.params.get("cdp_policy")
    mcp_policy = module.params.get("mcp_policy")
    lldp_policy = module.params.get("lldp_policy")
    stp_interface_policy = module.params.get("stp_interface_policy")
    egress_data_plane_policing_policy = module.params.get("egress_data_plane_policing_policy")
    ingress_data_plane_policing_policy = module.params.get("ingress_data_plane_policing_policy")
    priority_flow_control_policy = module.params.get("priority_flow_control_policy")
    fibre_channel_interface_policy = module.params.get("fibre_channel_interface_policy")
    slow_drain_policy = module.params.get("slow_drain_policy")
    port_channel_policy = module.params.get("port_channel_policy")
    monitoring_policy = module.params.get("monitoring_policy")
    storm_control_interface_policy = module.params.get("storm_control_interface_policy")
    l2_interface_policy = module.params.get("l2_interface_policy")
    port_security_policy = module.params.get("port_security_policy")
    link_flap_policy = module.params.get("link_flap_policy")
    link_level_flow_control = module.params.get("link_level_flow_control")
    mac_sec_interface_policy = module.params.get("mac_sec_interface_policy")
    copp_policy = module.params.get("copp_policy")
    poe_interface_policy = module.params.get("poe_interface_policy")
    port_authentication = module.params.get("port_authentication")
    dwdm = module.params.get("dwdm")
    aep = module.params.get("aep")
    transceiver_policy = module.params.get("transceiver_policy")
    sync_e_interface_policy = module.params.get("sync_e_interface_policy")

    state = module.params.get("state")
    name_alias = module.params.get("name_alias")

    aci = ACIModule(module)
    if lag_type == "leaf" and port_channel_policy is not None:
        aci.fail_json(
            "port_channel_policy is not a valid parameter for leaf\
 (leaf access port policy group), if used\
 assign null to it (port_channel_policy: null)."
        )
    invalid_parameters = {
        "transceiver_policy": "transceiver_policy is not a valid parameter for link/node (Port Channel, Virtual Port Channel),\
 if used assign null to it (transceiver_policy: null).",
        "port_authentication": "port_authentication is not a valid parameter for link/node (Port Channel, Virtual Port Channel),\
 if used assign null to it (port_authentication: null).",
        "dwdm": "dwdm is not a valid parameter for link/node (Port Channel, Virtual Port Channel),\
 if used assign null to it (dwdm: null).",
        "poe_interface_policy": "poe_interface_policy is not a valid parameter for link/node (Port Channel, Virtual Port Channel),\
 if used assign null to it (poe_interface_policy: null).",
    }

    if lag_type in ["link", "node"]:
        for param, message in invalid_parameters.items():
            if locals().get(param) is not None:
                aci.fail_json(message)

    if lag_type == "leaf":
        aci_class_name = "infraAccPortGrp"
        dn_name = "accportgrp"
        class_config_dict = dict(
            name=policy_group,
            descr=description,
            nameAlias=name_alias,
        )
        # Reset for target_filter
        lag_type = None
    else:
        aci_class_name = "infraAccBndlGrp"
        dn_name = "accbundle"
        class_config_dict = dict(
            name=policy_group,
            descr=description,
            lagT=lag_type,
            nameAlias=name_alias,
        )

    child_configs = [
        dict(
            infraRsCdpIfPol=dict(
                attributes=dict(
                    tnCdpIfPolName=cdp_policy,
                ),
            ),
        ),
        dict(
            infraRsFcIfPol=dict(
                attributes=dict(
                    tnFcIfPolName=fibre_channel_interface_policy,
                ),
            ),
        ),
        dict(
            infraRsHIfPol=dict(
                attributes=dict(
                    tnFabricHIfPolName=link_level_policy,
                ),
            ),
        ),
        dict(
            infraRsL2IfPol=dict(
                attributes=dict(
                    tnL2IfPolName=l2_interface_policy,
                ),
            ),
        ),
        dict(
            infraRsL2PortSecurityPol=dict(
                attributes=dict(
                    tnL2PortSecurityPolName=port_security_policy,
                ),
            ),
        ),
        dict(
            infraRsLacpPol=dict(
                attributes=dict(
                    tnLacpLagPolName=port_channel_policy,
                ),
            ),
        ),
        dict(
            infraRsLldpIfPol=dict(
                attributes=dict(
                    tnLldpIfPolName=lldp_policy,
                ),
            ),
        ),
        dict(
            infraRsMcpIfPol=dict(
                attributes=dict(
                    tnMcpIfPolName=mcp_policy,
                ),
            ),
        ),
        dict(
            infraRsMonIfInfraPol=dict(
                attributes=dict(
                    tnMonInfraPolName=monitoring_policy,
                ),
            ),
        ),
        dict(
            infraRsQosEgressDppIfPol=dict(
                attributes=dict(
                    tnQosDppPolName=egress_data_plane_policing_policy,
                ),
            ),
        ),
        dict(
            infraRsQosIngressDppIfPol=dict(
                attributes=dict(
                    tnQosDppPolName=ingress_data_plane_policing_policy,
                ),
            ),
        ),
        dict(
            infraRsQosPfcIfPol=dict(
                attributes=dict(
                    tnQosPfcIfPolName=priority_flow_control_policy,
                ),
            ),
        ),
        dict(
            infraRsQosSdIfPol=dict(
                attributes=dict(
                    tnQosSdIfPolName=slow_drain_policy,
                ),
            ),
        ),
        dict(
            infraRsStormctrlIfPol=dict(
                attributes=dict(
                    tnStormctrlIfPolName=storm_control_interface_policy,
                ),
            ),
        ),
        dict(
            infraRsStpIfPol=dict(
                attributes=dict(
                    tnStpIfPolName=stp_interface_policy,
                ),
            ),
        ),
        dict(
            infraRsLinkFlapPol=dict(
                attributes=dict(
                    tnFabricLinkFlapPolName=link_flap_policy,
                ),
            ),
        ),
        dict(
            infraRsQosLlfcIfPol=dict(
                attributes=dict(
                    tnQosLlfcIfPolName=link_level_flow_control,
                ),
            ),
        ),
        dict(
            infraRsMacsecIfPol=dict(
                attributes=dict(
                    tnMacsecIfPolName=mac_sec_interface_policy,
                ),
            ),
        ),
        dict(
            infraRsCoppIfPol=dict(
                attributes=dict(
                    tnCoppIfPolName=copp_policy,
                ),
            ),
        ),
    ]

    child_classes = [
        "infraRsAttEntP",
        "infraRsCdpIfPol",
        "infraRsFcIfPol",
        "infraRsHIfPol",
        "infraRsL2IfPol",
        "infraRsL2PortSecurityPol",
        "infraRsLacpPol",
        "infraRsLldpIfPol",
        "infraRsMcpIfPol",
        "infraRsMonIfInfraPol",
        "infraRsQosEgressDppIfPol",
        "infraRsQosIngressDppIfPol",
        "infraRsQosPfcIfPol",
        "infraRsQosSdIfPol",
        "infraRsStormctrlIfPol",
        "infraRsStpIfPol",
        "infraRsLinkFlapPol",
        "infraRsQosLlfcIfPol",
        "infraRsMacsecIfPol",
        "infraRsCoppIfPol",
    ]

    # Add infraRsattEntP binding only when aep is defined
    if aep is not None:
        child_configs.append(
            dict(
                infraRsAttEntP=dict(
                    attributes=dict(
                        tDn="uni/infra/attentp-{0}".format(aep),
                    ),
                ),
            )
        )

    # Add infraRsSynceEthIfPol/infraRsSynceEthIfPolBndlGrp binding only when sync_e_interface_policy is defined
    if sync_e_interface_policy is not None:
        if lag_type is None:
            child_configs.append(
                dict(
                    infraRsSynceEthIfPol=dict(
                        attributes=dict(
                            tnSynceEthIfPolName=sync_e_interface_policy,
                        ),
                    ),
                )
            )
            child_classes.append("infraRsSynceEthIfPol")
        elif lag_type == "node" or lag_type == "link":
            child_configs.append(
                dict(
                    infraRsSynceEthIfPolBndlGrp=dict(
                        attributes=dict(
                            tnSynceEthIfPolName=sync_e_interface_policy,
                        ),
                    ),
                )
            )
            child_classes.append("infraRsSynceEthIfPolBndlGrp")

    # Add the children only when lag_type == leaf (Leaf Interface specific policies).
    if lag_type is None:
        # Add infraRsOpticsIfPol binding only when transceiver_policy was defined
        if transceiver_policy is not None:
            child_configs.append(
                dict(
                    infraRsOpticsIfPol=dict(
                        attributes=dict(
                            tDn="uni/infra/{0}-{1}".format(transceiver_policy.get("type"), transceiver_policy.get("name")),
                        ),
                    ),
                )
            )
            child_classes.append("infraRsOpticsIfPol")
        if dwdm is not None:
            child_configs.append(
                dict(
                    infraRsDwdmIfPol=dict(
                        attributes=dict(
                            tnDwdmIfPolName=dwdm,
                        ),
                    ),
                )
            )
            child_classes.append("infraRsDwdmIfPol")
        if port_authentication is not None:
            child_configs.append(
                dict(
                    infraRsL2PortAuthPol=dict(
                        attributes=dict(
                            tnL2PortAuthPolName=port_authentication,
                        ),
                    ),
                )
            )
            child_classes.append("infraRsL2PortAuthPol")
        if poe_interface_policy is not None:
            child_configs.append(
                dict(
                    infraRsPoeIfPol=dict(
                        attributes=dict(
                            tnPoeIfPolName=poe_interface_policy,
                        ),
                    ),
                )
            )
            child_classes.append("infraRsPoeIfPol")

    aci.construct_url(
        root_class=dict(
            aci_class=aci_class_name,
            aci_rn="infra/funcprof/{0}-{1}".format(dn_name, policy_group),
            module_object=policy_group,
            target_filter={"name": policy_group, "lagT": lag_type},
        ),
        child_classes=child_classes,
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class=aci_class_name,
            class_config=class_config_dict,
            child_configs=child_configs,
        )

        aci.get_diff(aci_class=aci_class_name)

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on the Docker SDK for Python (https://github.com/docker/docker-py/)
#
# Copyright (c) 2016-2022 Docker, Inc.
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


class StoreError(RuntimeError):
    pass


class CredentialsNotFound(StoreError):
    pass


class InitializationError(StoreError):
    pass


def process_store_error(cpe, program):
    message = cpe.output.decode('utf-8')
    if 'credentials not found in native keychain' in message:
        return CredentialsNotFound(
            'No matching credentials in {0}'.format(
                program
            )
        )
    return StoreError(
        'Credentials store {0} exited with "{1}".'.format(
            program, cpe.output.decode('utf-8').strip()
        )
    )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2020, Lionel Hercot (@lhercot) <lhercot@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_version
short_description: Get version of MSO
description:
- Retrieve the code version of Cisco Multi-Site Orchestrator.
author:
- Lionel Hercot (@lhercot)
options:
  state:
    description:
    - Use C(query) for retrieving the version object.
    type: str
    choices: [ query ]
    default: query
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Get MSO version
  cisco.mso.mso_version:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(state=dict(type="str", default="query", choices=["query"]))

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    mso = MSOModule(module)

    path = "platform/version"

    # Query for mso.existing object
    mso.existing = mso.query_obj(path)
    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_l3out_extsubnet
short_description: Manage External Subnet objects (l3extSubnet:extsubnet)
description:
- Manage External Subnet objects (l3extSubnet:extsubnet)
options:
  tenant:
    description:
    - Name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
    required: true
  l3out:
    description:
    - Name of an existing L3Out.
    type: str
    aliases: [ l3out_name ]
    required: true
  extepg:
    description:
    - Name of an existing ExtEpg.
    type: str
    aliases: [ extepg_name ]
    required: true
  network:
    description:
    - The network address for the Subnet.
    type: str
    aliases: [ address, ip ]
  subnet_name:
    description:
    - Name of External Subnet being created.
    type: str
    aliases: [ name ]
  description:
    description:
    - Description for the External Subnet.
    type: str
    aliases: [ descr ]
  scope:
    description:
    - Determines the scope of the Subnet.
    - The C(export-rtctrl) option controls which external networks are advertised out of the fabric using route-maps and IP prefix-lists.
    - The C(import-rtctrl) option controls which external networks are advertised in to the fabric using route-maps and IP prefix-lists.
    - The C(import-security) option classifies for the external EPG.
      The rules and contracts defined in this external EPG apply to networks matching this subnet.
    - The C(shared-rtctrl) option controls which external prefixes are advertised to other tenants for shared services.
    - The C(shared-security) option configures the classifier for the subnets in the VRF where the routes are leaked.
    - The APIC defaults to C(import-security) when unset during creation.
    - The C(import-rtctrl) is only supported for BGP and OSPF.
    type: list
    elements: str
    choices: [ export-rtctrl, import-rtctrl, import-security, shared-rtctrl, shared-security ]
  aggregate:
    description:
    - Determines the Aggregate Routes for the Subnet.
    - The C(export-rtctrl) option to export all transit routes of a VRF (0/0 subnets).
    - The C(import-rtctrl) option to import all incoming routes of given L3 peers (0/0 subnets).
    - The C(shared-rtctrl) option to share routes learned in one VRF which needs to be advertised to another VRF.
      0/0 can be used to share all subnet routes across multiple VRFs.
    - The C(import-rtctrl) is only supported for BGP and OSPF.
    - Aggregate import route control is only available if the L3Out has 'Import Route Control Enforcement' enabled.
      Default this is disabled, M(cisco.aci.aci_l3out) with C(route_control) can be used to enable.
    type: list
    elements: str
    choices: [ export-rtctrl, import-rtctrl, shared-rtctrl]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- The C(tenant) and C(domain) and C(vrf) used must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant) and M(cisco.aci.aci_domain) and M(cisco.aci.aci_vrf) modules can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- module: cisco.aci.aci_domain
- module: cisco.aci.aci_vrf
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(l3ext:Out).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Rostyslav Davydenko (@rost-d)
- Cindy Zhao (@cizhao)
"""

EXAMPLES = r"""
- name: Add a new External Subnet
  cisco.aci.aci_l3out_extsubnet:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    l3out: prod_l3out
    extepg: prod_extepg
    description: External Subnet for Production ExtEpg
    network: 192.0.2.0/24
    scope: export-rtctrl
    aggregate: export-rtctrl
    state: present
  delegate_to: localhost

- name: Delete External Subnet
  cisco.aci.aci_l3out_extsubnet:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    l3out: prod_l3out
    extepg: prod_extepg
    network: 192.0.2.0/24
    state: absent
  delegate_to: localhost

- name: Query ExtEpg Subnet information
  cisco.aci.aci_l3out_extsubnet:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: production
    l3out: prod_l3out
    extepg: prod_extepg
    network: 192.0.2.0/24
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        tenant=dict(type="str", required=True, aliases=["tenant_name"]),
        l3out=dict(type="str", required=True, aliases=["l3out_name"]),
        extepg=dict(type="str", required=True, aliases=["extepg_name", "name"]),
        network=dict(type="str", aliases=["address", "ip"]),
        description=dict(type="str", aliases=["descr"]),
        subnet_name=dict(type="str", aliases=["name"]),
        scope=dict(type="list", elements="str", choices=["import-security", "export-rtctrl", "import-rtctrl", "shared-rtctrl", "shared-security"]),
        aggregate=dict(type="list", elements="str", choices=["export-rtctrl", "import-rtctrl", "shared-rtctrl"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        name_alias=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "present", ["network"]],
            ["state", "absent", ["network"]],
        ],
        required_by={"aggregate": "scope"},
    )

    aci = ACIModule(module)

    tenant = module.params.get("tenant")
    l3out = module.params.get("l3out")
    extepg = module.params.get("extepg")
    network = module.params.get("network")
    description = module.params.get("description")
    subnet_name = module.params.get("subnet_name")
    scope = module.params.get("scope")
    aggregate = module.params.get("aggregate")
    state = module.params.get("state")
    name_alias = module.params.get("name_alias")

    # Validation rule to only allow aggregate choice when there is a match in scope choice
    if aggregate and not set(aggregate).issubset(scope):
        aci.module.fail_json(msg="All aggregate values {0} need to be defined in scope {1}.".format(aggregate, scope))

    class_config = dict(ip=network, descr=description, name=subnet_name, nameAlias=name_alias)
    if scope:
        class_config["scope"] = ",".join(sorted(scope))
    if aggregate:
        class_config["aggregate"] = ",".join(sorted(aggregate))

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="l3extOut",
            aci_rn="out-{0}".format(l3out),
            module_object=l3out,
            target_filter={"name": l3out},
        ),
        subclass_2=dict(
            aci_class="l3extInstP",
            aci_rn="instP-{0}".format(extepg),
            module_object=extepg,
            target_filter={"name": extepg},
        ),
        subclass_3=dict(
            aci_class="l3extSubnet",
            aci_rn="extsubnet-[{0}]".format(network),
            module_object=network,
            target_filter={"name": network},
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(aci_class="l3extSubnet", class_config=class_config)

        aci.get_diff(aci_class="l3extSubnet")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Cisco and/or its affiliates.
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The nxos telemetry fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.telemetry.telemetry import (
    TelemetryArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.cmdref.telemetry.telemetry import (
    TMS_DESTGROUP,
    TMS_GLOBAL,
    TMS_SENSORGROUP,
    TMS_SUBSCRIPTION,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import NxosCmdRef
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.utils.telemetry.telemetry import (
    cr_key_lookup,
    get_instance_data,
    normalize_data,
)


class TelemetryFacts(object):
    """The nxos telemetry fact class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = TelemetryArgs.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for telemetry
        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf
        :rtype: dictionary
        :returns: facts
        """
        if connection:  # just for linting purposes, remove
            pass

        cmd_ref = {}
        cmd_ref["TMS_GLOBAL"] = {}
        cmd_ref["TMS_DESTGROUP"] = {}
        cmd_ref["TMS_SENSORGROUP"] = {}
        cmd_ref["TMS_SUBSCRIPTION"] = {}

        # For fact gathering, module state should be 'present' when using
        # NxosCmdRef to query state
        if self._module.params.get("state"):
            saved_module_state = self._module.params["state"]
            self._module.params["state"] = "present"

        # Get Telemetry Global Data
        cmd_ref["TMS_GLOBAL"]["ref"] = []
        cmd_ref["TMS_GLOBAL"]["ref"].append(NxosCmdRef(self._module, TMS_GLOBAL))
        ref = cmd_ref["TMS_GLOBAL"]["ref"][0]
        ref.set_context()
        ref.get_existing()
        device_cache = ref.cache_existing

        if device_cache is None:
            device_cache_lines = []
        else:
            device_cache_lines = device_cache.split("\n")

        # Get Telemetry Destination Group Data
        cmd_ref["TMS_DESTGROUP"]["ref"] = []
        for line in device_cache_lines:
            if re.search(r"destination-group", line):
                resource_key = line.strip()
                cmd_ref["TMS_DESTGROUP"]["ref"].append(NxosCmdRef(self._module, TMS_DESTGROUP))
                ref = cmd_ref["TMS_DESTGROUP"]["ref"][-1]
                ref.set_context([resource_key])
                ref.get_existing(device_cache)
                normalize_data(ref)

        # Get Telemetry Sensorgroup Group Data
        cmd_ref["TMS_SENSORGROUP"]["ref"] = []
        for line in device_cache_lines:
            if re.search(r"sensor-group", line):
                resource_key = line.strip()
                cmd_ref["TMS_SENSORGROUP"]["ref"].append(NxosCmdRef(self._module, TMS_SENSORGROUP))
                ref = cmd_ref["TMS_SENSORGROUP"]["ref"][-1]
                ref.set_context([resource_key])
                ref.get_existing(device_cache)

        # Get Telemetry Subscription Data
        cmd_ref["TMS_SUBSCRIPTION"]["ref"] = []
        for line in device_cache_lines:
            if re.search(r"subscription", line):
                resource_key = line.strip()
                cmd_ref["TMS_SUBSCRIPTION"]["ref"].append(
                    NxosCmdRef(self._module, TMS_SUBSCRIPTION),
                )
                ref = cmd_ref["TMS_SUBSCRIPTION"]["ref"][-1]
                ref.set_context([resource_key])
                ref.get_existing(device_cache)

        objs = []
        objs = self.render_config(self.generated_spec, cmd_ref)
        facts = {"telemetry": {}}
        if objs:
            # params = utils.validate_config(self.argument_spec, {'config': objs})
            facts["telemetry"] = objs

        ansible_facts["ansible_network_resources"].update(facts)
        if self._module.params.get("state"):
            self._module.params["state"] = saved_module_state
        return ansible_facts

    def render_config(self, spec, cmd_ref):
        """
        Render config as dictionary structure and delete keys
          from spec for null values

        :param spec: The facts tree, generated from the argspec
        :param conf: The configuration
        :rtype: dictionary
        :returns: The generated config
        """
        config = deepcopy(spec)
        config["destination_groups"] = []
        config["sensor_groups"] = []
        config["subscriptions"] = []
        managed_objects = [
            "TMS_GLOBAL",
            "TMS_DESTGROUP",
            "TMS_SENSORGROUP",
            "TMS_SUBSCRIPTION",
        ]

        # Walk the argspec and cmd_ref objects and build out config dict.
        for key in config.keys():
            for mo in managed_objects:
                for cr in cmd_ref[mo]["ref"]:
                    cr_keys = cr_key_lookup(key, mo)
                    for cr_key in cr_keys:
                        if cr._ref.get(cr_key) and cr._ref[cr_key].get("existing"):
                            if isinstance(config[key], dict):
                                for k in config[key].keys():
                                    for existing_key in cr._ref[cr_key]["existing"].keys():
                                        config[key][k] = cr._ref[cr_key]["existing"][existing_key][
                                            k
                                        ]
                                continue
                            if isinstance(config[key], list):
                                for existing_key in cr._ref[cr_key]["existing"].keys():
                                    data = get_instance_data(key, cr_key, cr, existing_key)
                                    config[key].append(data)
                                continue
                            for existing_key in cr._ref[cr_key]["existing"].keys():
                                config[key] = cr._ref[cr_key]["existing"][existing_key]
                        elif cr._ref.get(cr_key):
                            data = get_instance_data(key, cr_key, cr, None)
                            if isinstance(config[key], list) and data not in config[key]:
                                config[key].append(data)

        return utils.remove_empties(config)



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Test plugin file for netaddr tests: ipv6
"""

from __future__ import absolute_import, division, print_function

from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddress_utils import (
    _need_ipaddress,
    ip_network,
)
from ansible_collections.ansible.utils.plugins.plugin_utils.base.utils import _validate_args


__metaclass__ = type

DOCUMENTATION = """
    name: ipv6
    author: Priyam Sahoo (@priyamsahoo)
    version_added: "2.2.0"
    short_description: Test if something is an IPv6 address or network
    description:
        - This plugin checks if the provided value is a valid host or network IP address with IPv6 addressing scheme
    options:
        ip:
            description:
            - A string that represents the value against which the test is going to be performed
            - 'For example: C(10.1.1.1), C(10.0.0.0/8), or C(fe80::216:3eff:fee4:16f3)'
            type: str
            required: True
    notes:
"""

EXAMPLES = r"""

#### Simple examples

- name: Check if fe80::216:3eff:fee4:16f3 is a valid IPv6 address
  ansible.builtin.set_fact:
    data: "{{ 'fe80::216:3eff:fee4:16f3' is ansible.utils.ipv6 }}"

# TASK [Check if fe80::216:3eff:fee4:16f3 is a valid IPv6 address] *************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }

- name: Check if 2001:db8:a::/64 is a valid IPv6 address
  ansible.builtin.set_fact:
    data: "{{ '2001:db8:a::/64' is ansible.utils.ipv6 }}"

# TASK [Check if 2001:db8:a::/64 is a valid IPv6 address] **********************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }

- name: Check if 192.169.1.250 is not a valid IPv6 address
  ansible.builtin.set_fact:
    data: "{{ '192.169.1.250' is not ansible.utils.ipv6 }}"

# TASK [Check if 192.169.1.250 is not a valid IPv6 address] ********************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }
"""

RETURN = """
  data:
    description:
      - If jinja test satisfies plugin expression C(true)
      - If jinja test does not satisfy plugin expression C(false)
"""


@_need_ipaddress
def _ipv6(ip):
    """Test if something in an IPv6 address or network"""

    params = {"ip": ip}
    _validate_args("ipv6", DOCUMENTATION, params)

    try:
        return ip_network(ip).version == 6
    except Exception:
        return False


class TestModule(object):
    """network jinja test"""

    test_map = {"ipv6": _ipv6}

    def tests(self):
        return self.test_map
